@@ -273,10 +273,6 @@ public:
 
   int unsafe_sync_write;
 
-  int file_stripe_unit;
-  int file_stripe_count;
-  int object_size;
-  int file_replication;
 public:
   entity_name_t get_myname() { return messenger->get_myname(); } 
   void sync_write_commit(Inode *in);
@@ -397,7 +393,7 @@ protected:
 
   Client(Messenger *m, MonClient *mc);
   ~Client();
-  void tear_down_cache();   
+  void tear_down_cache();
 
   client_t get_nodeid() { return whoami; }
 
@@ -456,6 +452,7 @@ protected:
   void _async_invalidate(Inode *in, int64_t off, int64_t len, bool keep_caps);
   void _release(Inode *in);
   bool _flush(Inode *in);
+  void _flush_range(Inode *in, int64_t off, uint64_t size);
   void _flushed(Inode *in);
   void flush_set_callback(ObjectCacher::ObjectSet *oset);
 
@@ -530,8 +527,8 @@ private:
   int _removexattr(Inode *in, const char *nm, int uid=-1, int gid=-1);
   int _open(Inode *in, int flags, mode_t mode, Fh **fhp, int uid=-1, int gid=-1);
   int _create(Inode *in, const char *name, int flags, mode_t mode, Inode **inp, Fh **fhp,
-      int stripe_unit, int stripe_count, int object_size, const char *data_pool,
-	      int uid=-1, int gid=-1);
+              int stripe_unit, int stripe_count, int object_size, const char *data_pool,
+	      bool *created = NULL, int uid=-1, int gid=-1);
   loff_t _lseek(Fh *fh, loff_t offset, int whence);
   int _read(Fh *fh, int64_t offset, uint64_t size, bufferlist *bl);
   int _write(Fh *fh, int64_t offset, uint64_t size, const char *buf);
@@ -658,11 +655,8 @@ public:
   // expose osdmap 
   int get_local_osd();
   int get_pool_replication(int64_t pool);
-
-  void set_default_file_stripe_unit(int stripe_unit);
-  void set_default_file_stripe_count(int count);
-  void set_default_object_size(int size);
-  void set_default_file_replication(int replication);
+  int64_t get_pool_id(const char *pool_name);
+  string get_pool_name(int64_t pool);
 
   int enumerate_layout(int fd, vector<ObjectExtent>& result,
 		       loff_t length, loff_t offset);
