@@ -45,6 +45,17 @@
 #undef generic_dout
 #undef dendl
 
+void string_to_vec(std::vector<std::string>& args, std::string argstr)
+{
+  istringstream iss(argstr);
+  while(iss) {
+    string sub;
+    iss >> sub;
+    if (sub == "") break;
+    args.push_back(sub);
+  }
+}
+
 void env_to_vec(std::vector<const char*>& args, const char *name)
 {
   if (!name)
@@ -78,18 +89,17 @@ void argv_to_vec(int argc, const char **argv,
     args.push_back(argv[i]);
 }
 
-void vec_to_argv(std::vector<const char*>& args,
-                 int& argc, const char **&argv)
+void vec_to_argv(const char *argv0, std::vector<const char*>& args,
+                 int *argc, const char ***argv)
 {
-  const char *myname = "asdf";
-  if (argc && argv)
-    myname = argv[0];
-  argv = (const char**)malloc(sizeof(char*) * argc);
-  argc = 1;
-  argv[0] = myname;
+  *argv = (const char**)malloc(sizeof(char*) * (args.size() + 1));
+  if (!*argv)
+    throw bad_alloc();
+  *argc = 1;
+  (*argv)[0] = argv0;
 
   for (unsigned i=0; i<args.size(); i++)
-    argv[argc++] = args[i];
+    (*argv)[(*argc)++] = args[i];
 }
 
 bool parse_ip_port_vec(const char *s, vector<entity_addr_t>& vec)
@@ -337,12 +347,38 @@ bool ceph_argparse_withlonglong(std::vector<const char*> &args,
   return true;
 }
 
+bool ceph_argparse_withfloat(std::vector<const char*> &args,
+	std::vector<const char*>::iterator &i, float *ret,
+	std::ostream *oss, ...)
+{
+  bool r;
+  va_list ap;
+  std::string str;
+  va_start(ap, oss);
+  r = va_ceph_argparse_witharg(args, i, &str, ap);
+  va_end(ap);
+  if (!r) {
+    return false;
+  }
+
+  std::string err;
+  float myret = strict_strtof(str.c_str(), &err);
+  *ret = myret;
+  if (!err.empty()) {
+    *oss << err;
+  }
+  return true;
+}
+
 CephInitParameters ceph_argparse_early_args
 	  (std::vector<const char*>& args, uint32_t module_type, int flags,
 	   std::string *cluster, std::string *conf_file_list)
 {
   CephInitParameters iparams(module_type);
   std::string val;
+
+  vector<const char *> orig_args = args;
+
   for (std::vector<const char*>::iterator i = args.begin(); i != args.end(); ) {
     if (strcmp(*i, "--") == 0) {
       /* Normally we would use ceph_argparse_double_dash. However, in this
@@ -364,17 +400,26 @@ CephInitParameters ceph_argparse_early_args
 	     (ceph_argparse_witharg(args, i, &val, "-i", (char*)NULL))) {
       iparams.name.set_id(val);
     }
-    else if (ceph_argparse_witharg(args, i, &val, "--id", (char*)NULL)) {
+    else if (ceph_argparse_witharg(args, i, &val, "--id", "--user", (char*)NULL)) {
       iparams.name.set_id(val);
     }
     else if (ceph_argparse_witharg(args, i, &val, "--name", "-n", (char*)NULL)) {
       if (!iparams.name.from_str(val)) {
-	cerr << "You must pass a string of the form TYPE.ID to "
-	  << "the --name option. Valid types are: "
-	  << EntityName::get_valid_types_as_str() << std::endl;
+	cerr << "error parsing '" << val << "': expected string of the form TYPE.ID, "
+	     << "valid types are: " << EntityName::get_valid_types_as_str()
+	     << std::endl;
 	_exit(1);
       }
     }
+    else if (ceph_argparse_flag(args, i, "--show_args", (char*)NULL)) {
+      cout << "args: ";
+      for (std::vector<const char *>::iterator ci = orig_args.begin(); ci != orig_args.end(); ++ci) {
+        if (ci != orig_args.begin())
+          cout << " ";
+        cout << *ci;
+      }
+      cout << std::endl;
+    }
     else {
       // ignore
       ++i;
@@ -386,16 +431,17 @@ CephInitParameters ceph_argparse_early_args
 static void generic_usage(bool is_server)
 {
   cout << "\
---conf/-c        Read configuration from the given configuration file\n\
--d               Run in foreground, log to stderr.\n\
--f               Run in foreground, log to usual location.\n\
---id/-i          set ID portion of my name\n\
---name/-n        set name (TYPE.ID)\n\
---version        show version and quit\n\
+  --conf/-c        Read configuration from the given configuration file\n\
+  --id/-i          set ID portion of my name\n\
+  --name/-n        set name (TYPE.ID)\n\
+  --version        show version and quit\n\
 " << std::endl;
 
   if (is_server) {
-    cout << "   --debug_ms N\n";
+    cout << "\
+  -d               Run in foreground, log to stderr.\n\
+  -f               Run in foreground, log to usual location.\n";
+    cout << "  --debug_ms N\n";
     cout << "        set message debug level (e.g. 1)\n";
   }
 }
