@@ -30,6 +30,8 @@
 #define CEPH_FEATURE_MSG_AUTH	    (1<<23)
 #define CEPH_FEATURE_RECOVERY_RESERVATION (1<<24)
 #define CEPH_FEATURE_CRUSH_TUNABLES2 (1<<25)
+#define CEPH_FEATURE_CREATEPOOLID   (1<<26)
+#define CEPH_FEATURE_REPLY_CREATE_INODE   (1<<27)
 #define CEPH_FEATURE_OSD_HBMSGS  (1<<28)
 
 /*
@@ -59,9 +61,11 @@
 	 CEPH_FEATURE_MON_NULLROUTE |	 \
 	 CEPH_FEATURE_MON_GV |		 \
 	 CEPH_FEATURE_BACKFILL_RESERVATION | \
-	 CEPH_FEATURE_MSG_AUTH |	 \
+	 CEPH_FEATURE_MSG_AUTH |	     \
 	 CEPH_FEATURE_RECOVERY_RESERVATION | \
 	 CEPH_FEATURE_CRUSH_TUNABLES2 |	     \
+	 CEPH_FEATURE_CREATEPOOLID |	     \
+	 CEPH_FEATURE_REPLY_CREATE_INODE |   \
 	 CEPH_FEATURE_OSD_HBMSGS)
 
 #define CEPH_FEATURES_SUPPORTED_DEFAULT  CEPH_FEATURES_ALL
