@@ -841,6 +841,20 @@ int ceph_get_file_stripe_unit(struct ceph_mount_info *cmount, int fh);
  */
 int ceph_get_file_pool(struct ceph_mount_info *cmount, int fh);
 
+/**
+ * Get the name of the pool a file is stored in,
+ *
+ * Write the name of the file's pool to the buffer.  If buflen is 0, return
+ * a suggested length for the buffer.
+ *
+ * @param cmount the ceph mount handle to use.
+ * @param fh the open file descriptor referring to the file
+ * @param buf buffer to store the name in
+ * @param buflen size of the buffer
+ * @returns length in bytes of the pool name, or -ERANGE if the buffer is not large enough.
+ */
+int ceph_get_file_pool_name(struct ceph_mount_info *cmount, int fh, char *buf, size_t buflen);
+
 /**
  * Get the file replication information.
  * @param cmount the ceph mount handle to use.
@@ -849,6 +863,24 @@ int ceph_get_file_pool(struct ceph_mount_info *cmount, int fh);
  */
 int ceph_get_file_replication(struct ceph_mount_info *cmount, int fh);
 
+/**
+ * Get the id of the named pool.
+ *
+ * @param cmount the ceph mount handle to use.
+ * @param pool_name the name of the pool.
+ * @returns the pool id, or a negative error code on failure.
+ */
+int ceph_get_pool_id(struct ceph_mount_info *cmount, const char *pool_name);
+
+/**
+ * Get the pool replication factor.
+ *
+ * @param cmount the ceph mount handle to use.
+ * @param pool_id the pool id to look up
+ * @returns the replication factor, or a negative error code on failure.
+ */
+int ceph_get_pool_replication(struct ceph_mount_info *cmount, int pool_id);
+
 /**
  * Get the OSD address where the primary copy of a file stripe is located.
  *
@@ -864,56 +896,23 @@ int ceph_get_file_replication(struct ceph_mount_info *cmount, int fh);
 int ceph_get_file_stripe_address(struct ceph_mount_info *cmount, int fd, loff_t offset,
 				 struct sockaddr_storage *addr, int naddr);
 
+/**
+ * Get the file layout stripe unit granularity.
+ * @param cmount the ceph mount handle.
+ * @returns the stripe unit granularity or a negative error code on failure.
+ */
+int ceph_get_stripe_unit_granularity(struct ceph_mount_info *cmount);
+
 /** @} filelayout */
 
 /**
- * @defgroup libcephfs_h_default_filelayout Control the Default File Layout.
- * Functions for setting and getting the default file layout for new files.
- *
- * @{
- */
-
-/**
- * Set the default stripe unit of files created in this mount.
- *
- * @param cmount the ceph mount handle to use.
- * @param stripe the stripe size to use
- * @returns 0 on success or a negative error code on failure.
+ * No longer available.  Do not use.
+ * These functions will return -EOPNOTSUPP.
  */
 int ceph_set_default_file_stripe_unit(struct ceph_mount_info *cmount, int stripe);
-
-/**
- * Set the default stripe count of files created in this mount.
- *
- * @param cmount the ceph mount handle to use.
- * @param count the stripe count to use
- * @returns 0 on success or a negative error code on failure.
- */
 int ceph_set_default_file_stripe_count(struct ceph_mount_info *cmount, int count);
-
-/**
- * Set the default object size to use for striping.
- *
- * @param cmount the ceph mount handle to use.
- * @param size the default size for objects
- * @returns 0 on success or a negative error code on failure
- */
 int ceph_set_default_object_size(struct ceph_mount_info *cmount, int size);
-
-/**
- * No longer available.  Do not use.
- * Set the default preferred placement group.  This option will return
- * -EOPNOTSUPP as it is no longer supported.
- */
 int ceph_set_default_preferred_pg(struct ceph_mount_info *cmount, int osd);
-
-/**
- * Set the default file replication factor.
- *
- * @param cmount the ceph mount handle to use.
- * @param replication the default replication factor for files in this filesystem.
- * @returns 0 on success or negative error code on failure.
- */
 int ceph_set_default_file_replication(struct ceph_mount_info *cmount, int replication);
 
 /**
