@@ -17,6 +17,7 @@
 #include <string.h>
 #include <string>
 
+#include "auth/Crypto.h"
 #include "client/Client.h"
 #include "include/cephfs/libcephfs.h"
 #include "common/Mutex.h"
@@ -122,8 +123,6 @@ public:
     }
     if (client) {
       client->shutdown();
-      delete client;
-      client = NULL;
     }
     if (messenger) {
       messenger->shutdown();
@@ -135,6 +134,10 @@ public:
       delete monclient;
       monclient = NULL;
     }
+    if (client) {
+      delete client;
+      client = NULL;
+    }
   }
 
   bool is_mounted()
@@ -222,10 +225,13 @@ extern "C" const char *ceph_version(int *pmajor, int *pminor, int *ppatch)
 
 extern "C" int ceph_create_with_context(struct ceph_mount_info **cmount, CephContext *cct)
 {
-  // Function-static variables are thread-safe in gcc and in the forthcoming C++ standard
-  static int nonce_seed = 0;
+  uint64_t nonce = 0;
+
+  // 6 bytes of random and 2 bytes of pid
+  get_random_bytes((char*)&nonce, sizeof(nonce));
+  nonce &= ~0xffff;
+  nonce |= (uint64_t)getpid();
 
-  uint64_t nonce = (uint64_t)++nonce_seed * 1000000ull + (uint64_t)getpid();
   *cmount = new struct ceph_mount_info(nonce, cct);
   return 0;
 }
@@ -471,6 +477,8 @@ extern "C" int ceph_symlink(struct ceph_mount_info *cmount, const char *existing
 extern "C" int ceph_stat(struct ceph_mount_info *cmount, const char *path,
 			 struct stat *stbuf)
 {
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
   return cmount->get_client()->stat(path, stbuf);
 }
 
@@ -556,6 +564,8 @@ extern "C" int ceph_chmod(struct ceph_mount_info *cmount, const char *path, mode
 }
 extern "C" int ceph_fchmod(struct ceph_mount_info *cmount, int fd, mode_t mode)
 {
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
   return cmount->get_client()->fchmod(fd, mode);
 }
 extern "C" int ceph_chown(struct ceph_mount_info *cmount, const char *path,
@@ -568,11 +578,15 @@ extern "C" int ceph_chown(struct ceph_mount_info *cmount, const char *path,
 extern "C" int ceph_fchown(struct ceph_mount_info *cmount, int fd,
 			   uid_t uid, gid_t gid)
 {
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
   return cmount->get_client()->fchown(fd, uid, gid);
 }
 extern "C" int ceph_lchown(struct ceph_mount_info *cmount, const char *path,
 			   uid_t uid, gid_t gid)
 {
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
   return cmount->get_client()->lchown(path, uid, gid);
 }
 
@@ -705,6 +719,25 @@ extern "C" int ceph_get_file_pool(struct ceph_mount_info *cmount, int fh)
   return l.fl_pg_pool;
 }
 
+extern "C" int ceph_get_file_pool_name(struct ceph_mount_info *cmount, int fh, char *buf, size_t len)
+{
+  struct ceph_file_layout l;
+  int r;
+
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
+  r = cmount->get_client()->describe_layout(fh, &l);
+  if (r < 0)
+    return r;
+  string name = cmount->get_client()->get_pool_name(l.fl_pg_pool);
+  if (len == 0)
+    return name.length();
+  if (name.length() > len)
+    return -ERANGE;
+  strncpy(buf, name.c_str(), len);
+  return name.length();
+}
+
 extern "C" int ceph_get_file_replication(struct ceph_mount_info *cmount, int fh)
 {
   struct ceph_file_layout l;
@@ -722,36 +755,28 @@ extern "C" int ceph_get_file_replication(struct ceph_mount_info *cmount, int fh)
 extern "C" int ceph_set_default_file_stripe_unit(struct ceph_mount_info *cmount,
 						 int stripe)
 {
-  if (!cmount->is_mounted())
-    return -ENOTCONN;
-  cmount->get_client()->set_default_file_stripe_unit(stripe);
-  return 0;
+  // this option no longer exists
+  return -EOPNOTSUPP;
 }
 
 extern "C" int ceph_set_default_file_stripe_count(struct ceph_mount_info *cmount,
 						  int count)
 {
-  if (!cmount->is_mounted())
-    return -ENOTCONN;
-  cmount->get_client()->set_default_file_stripe_unit(count);
-  return 0;
+  // this option no longer exists
+  return -EOPNOTSUPP;
 }
 
 extern "C" int ceph_set_default_object_size(struct ceph_mount_info *cmount, int size)
 {
-  if (!cmount->is_mounted())
-    return -ENOTCONN;
-  cmount->get_client()->set_default_object_size(size);
-  return 0;
+  // this option no longer exists
+  return -EOPNOTSUPP;
 }
 
 extern "C" int ceph_set_default_file_replication(struct ceph_mount_info *cmount,
 						 int replication)
 {
-  if (!cmount->is_mounted())
-    return -ENOTCONN;
-  cmount->get_client()->set_default_file_replication(replication);
-  return 0;
+  // this option no longer exists
+  return -EOPNOTSUPP;
 }
 
 extern "C" int ceph_set_default_preferred_pg(struct ceph_mount_info *cmount, int osd)
@@ -816,3 +841,34 @@ extern "C" int ceph_debug_get_file_caps(struct ceph_mount_info *cmount, const ch
     return -ENOTCONN;
   return cmount->get_client()->get_caps_issued(path);
 }
+
+extern "C" int ceph_get_stripe_unit_granularity(struct ceph_mount_info *cmount)
+{
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
+  return CEPH_MIN_STRIPE_UNIT;
+}
+
+extern "C" int ceph_get_pool_id(struct ceph_mount_info *cmount, const char *pool_name)
+{
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
+
+  if (!pool_name || !pool_name[0])
+    return -EINVAL;
+
+  /* negative range reserved for errors */
+  int64_t pool_id = cmount->get_client()->get_pool_id(pool_name);
+  if (pool_id > 0x7fffffff)
+    return -ERANGE;
+
+  /* get_pool_id error codes fit in int */
+  return (int)pool_id;
+}
+
+extern "C" int ceph_get_pool_replication(struct ceph_mount_info *cmount, int pool_id)
+{
+  if (!cmount->is_mounted())
+    return -ENOTCONN;
+  return cmount->get_client()->get_pool_replication(pool_id);
+}
