@@ -324,6 +324,8 @@ inline ostream& operator<<(ostream &out, const vinodeno_t &vino) {
 struct byte_range_t {
   uint64_t first, last;    // interval client can write to
 
+  byte_range_t() : first(0), last(0) {}
+
   void encode(bufferlist &bl) const {
     ::encode(first, bl);
     ::encode(last, bl);
@@ -421,7 +423,7 @@ struct inode_t {
 	      truncate_pending(0),
 	      time_warp_seq(0),
 	      version(0), file_data_version(0), xattr_version(0), last_renamed_version(0) { 
-    clear_layout();
+    memset(&layout, 0, sizeof(layout));
     memset(&dir_layout, 0, sizeof(dir_layout));
   }
 
@@ -441,19 +443,6 @@ struct inode_t {
     truncate_pending++;
   }
 
-  bool has_layout() const {
-    // why on earth is there no converse of memchr() in string.h?
-    const char *p = (const char *)&layout;
-    for (size_t i = 0; i < sizeof(layout); i++)
-      if (p[i] != '\0')
-	return true;
-    return false;
-  }
-
-  void clear_layout() {
-    memset(&layout, 0, sizeof(layout));
-  }
-
   uint64_t get_layout_size_increment() {
     return (uint64_t)layout.fl_object_size * (uint64_t)layout.fl_stripe_count;
   }
