@@ -60,9 +60,9 @@ void MDSMonitor::create_new_fs(MDSMap &m, int metadata_pool, int data_pool)
 {
   m.max_mds = g_conf->max_mds;
   m.created = ceph_clock_now(g_ceph_context);
-  m.data_pg_pools.push_back(data_pool);
-  m.metadata_pg_pool = metadata_pool;
-  m.cas_pg_pool = -1;
+  m.data_pools.insert(data_pool);
+  m.metadata_pool = metadata_pool;
+  m.cas_pool = -1;
   m.compat = get_mdsmap_compat_set();
 
   m.session_timeout = g_conf->mds_session_timeout;
@@ -508,27 +508,6 @@ void MDSMonitor::dump_info(Formatter *f)
   f->close_section();
 }
 
-int MDSMonitor::parse_mds_id(const char *s, stringstream *pss)
-{
-  // osd.NNN?
-  if (strncmp(s, "mds.", 4) == 0) {
-    s += 4;
-  }
-
-  // NNN?
-  ostringstream ss;
-  long id = parse_pos_long(s, &ss);
-  if (id < 0) {
-    *pss << ss.str();
-    return id;
-  }
-  if (id > 0xffff) {
-    *pss << "mds id " << id << " is too large";
-    return -ERANGE;
-  }
-  return id;
-}
-
 bool MDSMonitor::preprocess_command(MMonCommand *m)
 {
   int r = -1;
@@ -560,8 +539,7 @@ bool MDSMonitor::preprocess_command(MMonCommand *m)
       for (std::vector<const char*>::iterator i = args.begin()+1; i != args.end(); ) {
 	if (ceph_argparse_double_dash(args, i))
 	  break;
-	else if (ceph_argparse_witharg_daemon(args, i, &val, "-f", "--format",
-					      (char*)NULL))
+	else if (ceph_argparse_witharg(args, i, &val, "-f", "--format", (char*)NULL))
 	  format = val;
 	else if (!epoch) {
 	  long l = parse_pos_long(*i++, &ss);
@@ -654,22 +632,18 @@ bool MDSMonitor::preprocess_command(MMonCommand *m)
 	}
       } else {
 	errno = 0;
-	int who = parse_mds_id(m->cmd[0].c_str(), &ss);
-	if (who < 0) {
-	  r = -EINVAL;
-	} else {
-	  m->cmd.erase(m->cmd.begin()); //done with target num now
-	  if (!errno && who >= 0) {
-	    if (mdsmap.is_up(who)) {
-	      mon->send_command(mdsmap.get_inst(who), m->cmd, paxos->get_version());
-	      r = 0;
-	      ss << "ok";
-	    } else {
-	      ss << "mds." << who << " no up";
-	      r = -ENOENT;
-	    }
+	int who = strtol(m->cmd[0].c_str(), 0, 10);
+	m->cmd.erase(m->cmd.begin()); //done with target num now
+	if (!errno && who >= 0) {
+	  if (mdsmap.is_up(who)) {
+	    mon->send_command(mdsmap.get_inst(who), m->cmd, paxos->get_version());
+	    r = 0;
+	    ss << "ok";
+	  } else {
+	    ss << "mds." << who << " no up";
+	    r = -ENOENT;
 	  }
-	}
+	} else ss << "specify mds number or *";
       }
     }
     else if (m->cmd[1] == "compat") {
@@ -700,9 +674,9 @@ bool MDSMonitor::preprocess_command(MMonCommand *m)
 
 int MDSMonitor::fail_mds(std::ostream &ss, const std::string &arg)
 {
-  stringstream ss2;
-  int w = parse_mds_id(arg.c_str(), &ss2);
-  if (w < 0) {
+  std::string err;
+  int w = strict_strtol(arg.c_str(), 10, &err);
+  if (!err.empty()) {
     // Try to interpret the arg as an MDS name
     const MDSMap::mds_info_t *mds_info = mdsmap.find_by_name(arg);
     if (!mds_info) {
@@ -941,14 +915,14 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
       int64_t poolid = parse_pos_long(m->cmd[2].c_str(), &ss);
       if (poolid < 0)
 	goto out;
-      pending_mdsmap.add_data_pg_pool(poolid);
+      pending_mdsmap.add_data_pool(poolid);
       ss << "added data pool " << poolid << " to mdsmap";
       r = 0;
     } else if (m->cmd[1] == "remove_data_pool" && m->cmd.size() == 3) {
       int64_t poolid = parse_pos_long(m->cmd[2].c_str(), &ss);
       if (poolid < 0)
 	goto out;
-      r = pending_mdsmap.remove_data_pg_pool(poolid);
+      r = pending_mdsmap.remove_data_pool(poolid);
       if (r == 0)
 	ss << "removed data pool " << poolid << " from mdsmap";
     } else if (m->cmd[1] == "newfs" && m->cmd.size() >= 4) {
