@@ -27,6 +27,7 @@ using namespace std;
 #include "include/Context.h"
 #include "include/types.h"
 #include "include/ceph_features.h"
+#include "auth/Crypto.h"
 
 #include <errno.h>
 #include <sstream>
@@ -438,6 +439,10 @@ public:
    * @param dest The entity to get a connection for.
    */
   virtual Connection *get_connection(const entity_inst_t& dest) = 0;
+  /**
+   * Get the Connection object associated with ourselves.
+   */
+  virtual Connection *get_loopback_connection() = 0;
   /**
    * Send a "keepalive" ping to the given dest, if it has a working Connection.
    * If the Messenger doesn't already have a Connection, or if the underlying
@@ -638,11 +643,11 @@ public:
    */
   bool ms_deliver_verify_authorizer(Connection *con, int peer_type,
 				    int protocol, bufferlist& authorizer, bufferlist& authorizer_reply,
-				    bool& isvalid) {
+				    bool& isvalid, CryptoKey& session_key) {
     for (list<Dispatcher*>::iterator p = dispatchers.begin();
 	 p != dispatchers.end();
 	 p++)
-      if ((*p)->ms_verify_authorizer(con, peer_type, protocol, authorizer, authorizer_reply, isvalid))
+      if ((*p)->ms_verify_authorizer(con, peer_type, protocol, authorizer, authorizer_reply, isvalid, session_key))
 	return true;
     return false;
   }
