@@ -25,6 +25,14 @@
 #include "common/debug.h"
 #include "common/errno.h"
 
+// Below included to get encode_encrypt(); That probably should be in Crypto.h, instead
+
+#include "auth/Crypto.h"
+#include "auth/cephx/CephxProtocol.h"
+#include "auth/AuthSessionHandler.h"
+
+// Constant to limit starting sequence number to 2^31.  Nothing special about it, just a big number.  PLR
+#define SEQ_MASK  0x7fffffff 
 #define dout_subsys ceph_subsys_ms
 
 #undef dout_prefix
@@ -32,6 +40,7 @@
 ostream& Pipe::_pipe_prefix(std::ostream *_dout) {
   return *_dout << "-- " << msgr->get_myinst().addr << " >> " << peer_addr << " pipe(" << this
 		<< " sd=" << sd << " :" << port
+		<< " s=" << state
 		<< " pgs=" << peer_global_seq
 		<< " cs=" << connect_seq
 		<< " l=" << policy.lossy
@@ -46,14 +55,18 @@ ostream& Pipe::_pipe_prefix(std::ostream *_dout) {
 
 Pipe::Pipe(SimpleMessenger *r, int st, Connection *con)
   : reader_thread(this), writer_thread(this),
+    delay_thread(NULL),
     msgr(r),
+    conn_id(r->dispatch_queue.get_id()),
     sd(-1), port(0),
     peer_type(-1),
     pipe_lock("SimpleMessenger::Pipe::pipe_lock"),
     state(st),
+    session_security(NULL),
     connection_state(NULL),
-    reader_running(false), reader_joining(false), writer_running(false),
-    in_q(r->dispatch_queue.create_queue(this)),
+    reader_running(false), reader_needs_join(false),
+    writer_running(false),
+    in_q(&(r->dispatch_queue)),
     keepalive(false),
     close_on_empty(false),
     connect_seq(0), peer_global_seq(0),
@@ -65,6 +78,12 @@ Pipe::Pipe(SimpleMessenger *r, int st, Connection *con)
     connection_state = new Connection();
     connection_state->pipe = get();
   }
+
+  if (randomize_out_seq()) {
+    lsubdout(msgr->cct,ms,15) << "Pipe(): Could not get random bytes to set seq number for session reset; set seq number to " << out_seq << dendl;
+  }
+    
+
   msgr->timeout = msgr->cct->_conf->ms_tcp_read_timeout * 1000; //convert to ms
   if (msgr->timeout == 0)
     msgr->timeout = -1;
@@ -72,11 +91,12 @@ Pipe::Pipe(SimpleMessenger *r, int st, Connection *con)
 
 Pipe::~Pipe()
 {
-  in_q->put();
   assert(out_q.empty());
   assert(sent.empty());
   if (connection_state)
     connection_state->put();
+  delete session_security;
+  delete delay_thread;
 }
 
 void Pipe::handle_ack(uint64_t seq)
@@ -102,10 +122,24 @@ void Pipe::start_reader()
 {
   assert(pipe_lock.is_locked());
   assert(!reader_running);
+  if (reader_needs_join) {
+    reader_thread.join();
+    reader_needs_join = false;
+  }
   reader_running = true;
   reader_thread.create(msgr->cct->_conf->ms_rwthread_stack_bytes);
 }
 
+void Pipe::maybe_start_delay_thread()
+{
+  if (!delay_thread &&
+      msgr->cct->_conf->ms_inject_delay_type.find(ceph_entity_type_name(connection_state->peer_type)) != string::npos) {
+    lsubdout(msgr->cct, ms, 1) << "setting up a delay queue on Pipe " << this << dendl;
+    delay_thread = new DelayedDelivery(this);
+    delay_thread->create();
+  }
+}
+
 void Pipe::start_writer()
 {
   assert(pipe_lock.is_locked());
@@ -118,24 +152,61 @@ void Pipe::join_reader()
 {
   if (!reader_running)
     return;
-  assert(!reader_joining);
-  reader_joining = true;
   cond.Signal();
   pipe_lock.Unlock();
   reader_thread.join();
   pipe_lock.Lock();
-  assert(reader_joining);
-  reader_joining = false;
+  reader_needs_join = false;
 }
 
-
-void Pipe::queue_received(Message *m, int priority)
+void Pipe::DelayedDelivery::discard()
 {
-  assert(pipe_lock.is_locked());
-  in_q->queue(m, priority);
+  lgeneric_subdout(pipe->msgr->cct, ms, 20) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::discard" << dendl;
+  Mutex::Locker l(delay_lock);
+  while (!delay_queue.empty()) {
+    Message *m = delay_queue.front().second;
+    pipe->msgr->dispatch_throttle_release(m->get_dispatch_throttle_size());
+    m->put();
+    delay_queue.pop_front();
+  }
 }
 
+void Pipe::DelayedDelivery::flush()
+{
+  lgeneric_subdout(pipe->msgr->cct, ms, 20) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::flush" << dendl;
+  Mutex::Locker l(delay_lock);
+  while (!delay_queue.empty()) {
+    Message *m = delay_queue.front().second;
+    delay_queue.pop_front();
+    pipe->in_q->enqueue(m, m->get_priority(), pipe->conn_id);
+  }
+}
 
+void *Pipe::DelayedDelivery::entry()
+{
+  Mutex::Locker locker(delay_lock);
+  lgeneric_subdout(pipe->msgr->cct, ms, 20) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::entry start" << dendl;
+
+  while (!stop_delayed_delivery) {
+    if (delay_queue.empty()) {
+      lgeneric_subdout(pipe->msgr->cct, ms, 30) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::entry sleeping on delay_cond because delay queue is empty" << dendl;
+      delay_cond.Wait(delay_lock);
+      continue;
+    }
+    utime_t release = delay_queue.front().first;
+    if (release > ceph_clock_now(pipe->msgr->cct)) {
+      lgeneric_subdout(pipe->msgr->cct, ms, 10) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::entry sleeping on delay_cond until " << release << dendl;
+      delay_cond.WaitUntil(delay_lock, release);
+      continue;
+    }
+    Message *m = delay_queue.front().second;
+    lgeneric_subdout(pipe->msgr->cct, ms, 10) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::entry dequeuing message " << m << " for delivery, past " << release << dendl;
+    delay_queue.pop_front();
+    pipe->in_q->enqueue(m, m->get_priority(), pipe->conn_id);
+  }
+  lgeneric_subdout(pipe->msgr->cct, ms, 20) << pipe->_pipe_prefix(_dout) << "DelayedDelivery::entry stop" << dendl;
+  return NULL;
+}
 
 int Pipe::accept()
 {
@@ -149,6 +220,7 @@ int Pipe::accept()
   if (rc < 0) {
     ldout(msgr->cct,10) << "accept couldn't write banner" << dendl;
     state = STATE_CLOSED;
+    state_closed.set(1);
     return -1;
   }
 
@@ -166,6 +238,7 @@ int Pipe::accept()
     char buf[80];
     ldout(msgr->cct,0) << "accept failed to getpeername " << errno << " " << strerror_r(errno, buf, sizeof(buf)) << dendl;
     state = STATE_CLOSED;
+    state_closed.set(1);
     return -1;
   }
   ::encode(socket_addr, addrs);
@@ -174,22 +247,25 @@ int Pipe::accept()
   if (rc < 0) {
     ldout(msgr->cct,10) << "accept couldn't write my+peer addr" << dendl;
     state = STATE_CLOSED;
+    state_closed.set(1);
     return -1;
   }
 
-  ldout(msgr->cct,1) << "accept sd=" << sd << dendl;
+  ldout(msgr->cct,1) << "accept sd=" << sd << " " << socket_addr << dendl;
   
   // identify peer
   char banner[strlen(CEPH_BANNER)+1];
   if (tcp_read(banner, strlen(CEPH_BANNER)) < 0) {
     ldout(msgr->cct,10) << "accept couldn't read banner" << dendl;
     state = STATE_CLOSED;
+    state_closed.set(1);
     return -1;
   }
   if (memcmp(banner, CEPH_BANNER, strlen(CEPH_BANNER))) {
     banner[strlen(CEPH_BANNER)] = 0;
     ldout(msgr->cct,1) << "accept peer sent bad banner '" << banner << "' (should be '" << CEPH_BANNER << "')" << dendl;
     state = STATE_CLOSED;
+    state_closed.set(1);
     return -1;
   }
   bufferlist addrbl;
@@ -200,6 +276,7 @@ int Pipe::accept()
   if (tcp_read(addrbl.c_str(), addrbl.length()) < 0) {
     ldout(msgr->cct,10) << "accept couldn't read peer_addr" << dendl;
     state = STATE_CLOSED;
+    state_closed.set(1);
     return -1;
   }
   {
@@ -226,6 +303,7 @@ int Pipe::accept()
   bool authorizer_valid;
   uint64_t feat_missing;
   bool replaced = false;
+  CryptoKey session_key;
 
   // this should roughly mirror pseudocode at
   //  http://ceph.newdream.net/wiki/Messaging_protocol
@@ -276,6 +354,24 @@ int Pipe::accept()
       goto reply;
     }
 
+    // require signatures for cephx?
+    if (connect.authorizer_protocol == CEPH_AUTH_CEPHX) {
+      if (peer_type == CEPH_ENTITY_TYPE_OSD ||
+	  peer_type == CEPH_ENTITY_TYPE_MDS) {
+	if (msgr->cct->_conf->cephx_require_signatures ||
+	    msgr->cct->_conf->cephx_cluster_require_signatures) {
+	  ldout(msgr->cct,10) << "using cephx, requiring MSG_AUTH feature bit for cluster" << dendl;
+	  policy.features_required |= CEPH_FEATURE_MSG_AUTH;
+	}
+      } else {
+	if (msgr->cct->_conf->cephx_require_signatures ||
+	    msgr->cct->_conf->cephx_service_require_signatures) {
+	  ldout(msgr->cct,10) << "using cephx, requiring MSG_AUTH feature bit for service" << dendl;
+	  policy.features_required |= CEPH_FEATURE_MSG_AUTH;
+	}
+      }
+    }
+
     feat_missing = policy.features_required & ~(uint64_t)connect.features;
     if (feat_missing) {
       ldout(msgr->cct,1) << "peer missing required features " << std::hex << feat_missing << std::dec << dendl;
@@ -285,21 +381,31 @@ int Pipe::accept()
     }
     
     msgr->lock.Unlock();
-    if (msgr->verify_authorizer(connection_state, peer_type,
-				connect.authorizer_protocol, authorizer, authorizer_reply, authorizer_valid) &&
+
+    // Check the authorizer.  If not good, bail out.
+
+    if (!msgr->verify_authorizer(connection_state, peer_type, connect.authorizer_protocol, authorizer, 
+				 authorizer_reply, authorizer_valid, session_key) ||
 	!authorizer_valid) {
-      ldout(msgr->cct,0) << "accept bad authorizer" << dendl;
+      ldout(msgr->cct,0) << "accept: got bad authorizer" << dendl;
       reply.tag = CEPH_MSGR_TAG_BADAUTHORIZER;
+      delete session_security;
+      session_security = NULL;
       goto reply;
-    }
+    } 
+
+    // We've verified the authorizer for this pipe, so set up the session security structure.  PLR
+
+    ldout(msgr->cct,10) << "accept:  setting up session_security." << dendl;
+
     msgr->lock.Lock();
     if (msgr->dispatch_queue.stop)
       goto shutting_down;
 
     
     // existing?
-    if (msgr->rank_pipe.count(peer_addr)) {
-      existing = msgr->rank_pipe[peer_addr];
+    existing = msgr->_lookup_pipe(peer_addr);
+    if (existing) {
       existing->pipe_lock.Lock();
 
       if (connect.global_seq < existing->peer_global_seq) {
@@ -462,17 +568,17 @@ int Pipe::accept()
 
     // make existing Connection reference us
     existing->connection_state->reset_pipe(this);
- 
+
+    // flush/queue any existing delayed messages
+    if (existing->delay_thread)
+      existing->delay_thread->flush();
+
     // steal incoming queue
+    uint64_t replaced_conn_id = conn_id;
+    conn_id = existing->conn_id;
+    existing->conn_id = replaced_conn_id;
     in_seq = existing->in_seq;
     in_seq_acked = in_seq;
-    in_q->put();
-    in_q = existing->in_q;
-    in_q->lock.Lock();
-    in_q->parent = this;
-    in_q->restart_queue();
-    in_q->lock.Unlock();
-    existing->in_q = msgr->dispatch_queue.create_queue(existing);
 
     // steal outgoing queue and out_seq
     existing->requeue_sent();
@@ -502,9 +608,13 @@ int Pipe::accept()
   if (policy.lossy)
     reply.flags = reply.flags | CEPH_MSG_CONNECT_LOSSY;
 
-  connection_state->set_features((int)reply.features & (int)connect.features);
+  connection_state->set_features((uint64_t)reply.features & (uint64_t)connect.features);
   ldout(msgr->cct,10) << "accept features " << connection_state->get_features() << dendl;
 
+  delete session_security;
+  session_security = get_auth_session_handler(msgr->cct, connect.authorizer_protocol, session_key,
+					      connection_state->get_features());
+
   // notify
   msgr->dispatch_queue.queue_accept(connection_state);
 
@@ -516,13 +626,13 @@ int Pipe::accept()
 
   rc = tcp_write((char*)&reply, sizeof(reply));
   if (rc < 0) {
-    goto fail_unlocked;
+    goto fail_registered;
   }
 
   if (reply.authorizer_len) {
     rc = tcp_write(authorizer_reply.c_str(), authorizer_reply.length());
     if (rc < 0) {
-      goto fail_unlocked;
+      goto fail_registered;
     }
   }
 
@@ -530,11 +640,11 @@ int Pipe::accept()
     uint64_t newly_acked_seq = 0;
     if(tcp_write((char*)&existing_seq, sizeof(existing_seq)) < 0) {
       ldout(msgr->cct,2) << "accept write error on in_seq" << dendl;
-      goto fail_unlocked;
+      goto fail_registered;
     }
     if (tcp_read((char*)&newly_acked_seq, sizeof(newly_acked_seq)) < 0) {
       ldout(msgr->cct,2) << "accept read error on newly_acked_seq" << dendl;
-      goto fail_unlocked;
+      goto fail_registered;
     }
     requeue_sent(newly_acked_seq);
   }
@@ -546,18 +656,34 @@ int Pipe::accept()
   }
   ldout(msgr->cct,20) << "accept done" << dendl;
   pipe_lock.Unlock();
+
+  maybe_start_delay_thread();
+
   return 0;   // success.
 
+ fail_registered:
+  ldout(msgr->cct, 10) << "accept fault after register" << dendl;
+
+  if (msgr->cct->_conf->ms_inject_internal_delays) {
+    ldout(msgr->cct, 10) << " sleep for " << msgr->cct->_conf->ms_inject_internal_delays << dendl;
+    utime_t t;
+    t.set_from_double(msgr->cct->_conf->ms_inject_internal_delays);
+    t.sleep();
+  }
+
  fail_unlocked:
   pipe_lock.Lock();
   if (state != STATE_CLOSED) {
     bool queued = is_queued();
-    if (queued)
+    ldout(msgr->cct, 10) << "  queued = " << (int)queued << dendl;
+    if (queued) {
       state = policy.server ? STATE_STANDBY : STATE_CONNECTING;
-    else if (replaced)
+    } else if (replaced) {
       state = STATE_STANDBY;
-    else
+    } else {
       state = STATE_CLOSED;
+      state_closed.set(1);
+    }
     fault();
     if (queued || replaced)
       start_writer();
@@ -567,8 +693,17 @@ int Pipe::accept()
 
  shutting_down:
   msgr->lock.Unlock();
+
+  if (msgr->cct->_conf->ms_inject_internal_delays) {
+    ldout(msgr->cct, 10) << " sleep for " << msgr->cct->_conf->ms_inject_internal_delays << dendl;
+    utime_t t;
+    t.set_from_double(msgr->cct->_conf->ms_inject_internal_delays);
+    t.sleep();
+  }
+
   pipe_lock.Lock();
   state = STATE_CLOSED;
+  state_closed.set(1);
   fault();
   pipe_lock.Unlock();
   return -1;
@@ -774,6 +909,13 @@ int Pipe::connect()
       }
     }
 
+    if (conf->ms_inject_internal_delays) {
+      ldout(msgr->cct, 10) << " sleep for " << msgr->cct->_conf->ms_inject_internal_delays << dendl;
+      utime_t t;
+      t.set_from_double(msgr->cct->_conf->ms_inject_internal_delays);
+      t.sleep();
+    }
+
     pipe_lock.Lock();
     if (state != STATE_CONNECTING) {
       ldout(msgr->cct,0) << "connect got RESETSESSION but no longer connecting" << dendl;
@@ -800,13 +942,13 @@ int Pipe::connect()
         goto stop_locked;
       got_bad_auth = true;
       pipe_lock.Unlock();
+      delete authorizer;
       authorizer = msgr->get_authorizer(peer_type, true);  // try harder
       continue;
     }
     if (reply.tag == CEPH_MSGR_TAG_RESETSESSION) {
       ldout(msgr->cct,0) << "connect got RESETSESSION" << dendl;
       was_session_reset();
-      in_q->restart_queue();
       cseq = 0;
       pipe_lock.Unlock();
       continue;
@@ -866,12 +1008,26 @@ int Pipe::connect()
       ldout(msgr->cct,10) << "connect success " << connect_seq << ", lossy = " << policy.lossy
 	       << ", features " << connection_state->get_features() << dendl;
       
+
+      // If we have an authorizer, get a new AuthSessionHandler to deal with ongoing security of the
+      // connection.  PLR
+
+      delete session_security;
+      if (authorizer != NULL) {
+        session_security = get_auth_session_handler(msgr->cct, authorizer->protocol, authorizer->session_key,
+						    connection_state->get_features());
+      }  else {
+        // We have no authorizer, so we shouldn't be applying security to messages in this pipe.  PLR
+	session_security = NULL;
+      }
+
       msgr->dispatch_queue.queue_connect(connection_state);
       
       if (!reader_running) {
 	ldout(msgr->cct,20) << "connect starting reader" << dendl;
 	start_reader();
       }
+      maybe_start_delay_thread();
       delete authorizer;
       return 0;
     }
@@ -882,10 +1038,17 @@ int Pipe::connect()
   }
 
  fail:
+  if (conf->ms_inject_internal_delays) {
+    ldout(msgr->cct, 10) << " sleep for " << msgr->cct->_conf->ms_inject_internal_delays << dendl;
+    utime_t t;
+    t.set_from_double(msgr->cct->_conf->ms_inject_internal_delays);
+    t.sleep();
+  }
+
   pipe_lock.Lock();
  fail_locked:
   if (state == STATE_CONNECTING)
-    fault(true);
+    fault();
   else
     ldout(msgr->cct,3) << "connect fault, but state = " << get_state_name()
 		       << " != connecting, stopping" << dendl;
@@ -899,17 +1062,18 @@ void Pipe::register_pipe()
 {
   ldout(msgr->cct,10) << "register_pipe" << dendl;
   assert(msgr->lock.is_locked());
-  assert(msgr->rank_pipe.count(peer_addr) == 0);
+  Pipe *existing = msgr->_lookup_pipe(peer_addr);
+  assert(existing == NULL);
   msgr->rank_pipe[peer_addr] = this;
 }
 
 void Pipe::unregister_pipe()
 {
   assert(msgr->lock.is_locked());
-  if (msgr->rank_pipe.count(peer_addr) &&
-      msgr->rank_pipe[peer_addr] == this) {
+  hash_map<entity_addr_t,Pipe*>::iterator p = msgr->rank_pipe.find(peer_addr);
+  if (p != msgr->rank_pipe.end() && p->second == this) {
     ldout(msgr->cct,10) << "unregister_pipe" << dendl;
-    msgr->rank_pipe.erase(peer_addr);
+    msgr->rank_pipe.erase(p);
   } else {
     ldout(msgr->cct,10) << "unregister_pipe - not registered" << dendl;
   }
@@ -924,14 +1088,17 @@ void Pipe::requeue_sent(uint64_t max_acked)
   list<Message*>& rq = out_q[CEPH_MSG_PRIO_HIGHEST];
   while (!sent.empty()) {
     Message *m = sent.back();
+    sent.pop_back();
     if (m->get_seq() > max_acked) {
-      sent.pop_back();
       ldout(msgr->cct,10) << "requeue_sent " << *m << " for resend seq " << out_seq
           << " (" << m->get_seq() << ")" << dendl;
       rq.push_front(m);
       out_seq--;
-    } else
-      sent.clear();
+    } else {
+      ldout(msgr->cct,10) << "requeue_sent " << *m << " for resend seq " << out_seq
+			  << " <= max_acked " << max_acked << ", discarding" << dendl;
+      m->put();
+    }
   }
 }
 
@@ -944,25 +1111,18 @@ void Pipe::discard_out_queue()
   ldout(msgr->cct,10) << "discard_queue" << dendl;
 
   for (list<Message*>::iterator p = sent.begin(); p != sent.end(); p++) {
-    if (*p < (void *) DispatchQueue::D_NUM_CODES) {
-      continue; // skip non-Message dispatch codes
-    }
     ldout(msgr->cct,20) << "  discard " << *p << dendl;
     (*p)->put();
   }
   sent.clear();
   for (map<int,list<Message*> >::iterator p = out_q.begin(); p != out_q.end(); p++)
     for (list<Message*>::iterator r = p->second.begin(); r != p->second.end(); r++) {
-      if (*r < (void *) DispatchQueue::D_NUM_CODES) {
-        continue; // skip non-Message dispatch codes
-      }
       ldout(msgr->cct,20) << "  discard " << *r << dendl;
       (*r)->put();
     }
   out_q.clear();
 }
 
-
 void Pipe::fault(bool onread)
 {
   const md_config_t *conf = msgr->cct->_conf;
@@ -986,19 +1146,30 @@ void Pipe::fault(bool onread)
   shutdown_socket();
 
   // lossy channel?
-  if (policy.lossy) {
+  if (policy.lossy && state != STATE_CONNECTING) {
     ldout(msgr->cct,10) << "fault on lossy channel, failing" << dendl;
 
     stop();
 
-    // ugh
+    // crib locks, blech.  note that Pipe is now STATE_CLOSED and the
+    // rank_pipe entry is ignored by others.
     pipe_lock.Unlock();
+
+    if (conf->ms_inject_internal_delays) {
+      ldout(msgr->cct, 10) << " sleep for " << msgr->cct->_conf->ms_inject_internal_delays << dendl;
+      utime_t t;
+      t.set_from_double(msgr->cct->_conf->ms_inject_internal_delays);
+      t.sleep();
+    }
+
     msgr->lock.Lock();
     pipe_lock.Lock();
     unregister_pipe();
     msgr->lock.Unlock();
 
-    in_q->discard_queue();
+    in_q->discard_queue(conn_id);
+    if (delay_thread)
+      delay_thread->discard();
     discard_out_queue();
 
     // disconnect from Connection, and mark it failed.  future messages
@@ -1010,6 +1181,10 @@ void Pipe::fault(bool onread)
     return;
   }
 
+  // queue delayed items immediately
+  if (delay_thread)
+    delay_thread->flush();
+
   // requeue sent items
   requeue_sent();
 
@@ -1017,7 +1192,7 @@ void Pipe::fault(bool onread)
     ldout(msgr->cct,0) << "fault with nothing to send, going to standby" << dendl;
     state = STATE_STANDBY;
     return;
-  } 
+  }
 
   if (state != STATE_CONNECTING) {
     if (policy.server) {
@@ -1042,17 +1217,38 @@ void Pipe::fault(bool onread)
   }
 }
 
+int Pipe::randomize_out_seq()
+{
+  if (connection_state->get_features() & CEPH_FEATURE_MSG_AUTH) {
+    // Set out_seq to a random value, so CRC won't be predictable.   Don't bother checking seq_error
+    // here.  We'll check it on the call.  PLR
+    int seq_error = get_random_bytes((char *)&out_seq, sizeof(out_seq));
+    out_seq &= SEQ_MASK;
+    lsubdout(msgr->cct, ms, 10) << "randomize_out_seq " << out_seq << dendl;
+    return seq_error;
+  } else {
+    // previously, seq #'s always started at 0.
+    out_seq = 0;
+    return 0;
+  }
+}
+
 void Pipe::was_session_reset()
 {
   assert(pipe_lock.is_locked());
 
   ldout(msgr->cct,10) << "was_session_reset" << dendl;
-  in_q->discard_queue();
+  in_q->discard_queue(conn_id);
+  if (delay_thread)
+    delay_thread->discard();
   discard_out_queue();
 
   msgr->dispatch_queue.queue_remote_reset(connection_state);
 
-  out_seq = 0;
+  if (randomize_out_seq()) {
+    lsubdout(msgr->cct,ms,15) << "was_session_reset(): Could not get random bytes to set seq number for session reset; set seq number to " << out_seq << dendl;
+  }
+
   in_seq = 0;
   connect_seq = 0;
 }
@@ -1062,6 +1258,7 @@ void Pipe::stop()
   ldout(msgr->cct,10) << "stop" << dendl;
   assert(pipe_lock.is_locked());
   state = STATE_CLOSED;
+  state_closed.set(1);
   cond.Signal();
   shutdown_socket();
 }
@@ -1166,16 +1363,29 @@ void Pipe::reader()
       ldout(msgr->cct,10) << "reader got message "
 	       << m->get_seq() << " " << m << " " << *m
 	       << dendl;
-      queue_received(m);
+
+      if (delay_thread) {
+	utime_t release;
+	if (rand() % 10000 < msgr->cct->_conf->ms_inject_delay_probability * 10000.0) {
+	  release = m->get_recv_stamp();
+	  release += msgr->cct->_conf->ms_inject_delay_max * (double)(rand() % 10000) / 10000.0;
+	  lsubdout(msgr->cct, ms, 1) << "queue_received will delay until " << release << " on " << m << " " << *m << dendl;
+	}
+	delay_thread->queue(release, m);
+      } else {
+	in_q->enqueue(m, m->get_priority(), conn_id);
+      }
     } 
     
     else if (tag == CEPH_MSGR_TAG_CLOSE) {
       ldout(msgr->cct,20) << "reader got CLOSE" << dendl;
       pipe_lock.Lock();
-      if (state == STATE_CLOSING)
+      if (state == STATE_CLOSING) {
 	state = STATE_CLOSED;
-      else
+	state_closed.set(1);
+      } else {
 	state = STATE_CLOSING;
+      }
       cond.Signal();
       break;
     }
@@ -1189,6 +1399,7 @@ void Pipe::reader()
  
   // reap?
   reader_running = false;
+  reader_needs_join = true;
   unlock_maybe_reap();
   ldout(msgr->cct,10) << "reader done" << dendl;
 }
@@ -1223,6 +1434,7 @@ void Pipe::writer()
       ldout(msgr->cct,20) << "writer writing CLOSE tag" << dendl;
       char tag = CEPH_MSGR_TAG_CLOSE;
       state = STATE_CLOSED;
+      state_closed.set(1);
       pipe_lock.Unlock();
       if (sd) {
 	int r = ::write(sd, &tag, 1);
@@ -1272,18 +1484,44 @@ void Pipe::writer()
 	  sent.push_back(m); 
 	  m->get();
 	}
-	pipe_lock.Unlock();
-
-        ldout(msgr->cct,20) << "writer encoding " << m->get_seq() << " " << m << " " << *m << dendl;
 
 	// associate message with Connection (for benefit of encode_payload)
 	m->set_connection(connection_state->get());
 
+        ldout(msgr->cct,20) << "writer encoding " << m->get_seq() << " " << m << " " << *m << dendl;
+
 	// encode and copy out of *m
 	m->encode(connection_state->get_features(), !msgr->cct->_conf->ms_nocrc);
 
+	// prepare everything
+	ceph_msg_header& header = m->get_header();
+	ceph_msg_footer& footer = m->get_footer();
+
+	// Now that we have all the crcs calculated, handle the
+	// digital signature for the message, if the pipe has session
+	// security set up.  Some session security options do not
+	// actually calculate and check the signature, but they should
+	// handle the calls to sign_message and check_signature.  PLR
+	if (session_security == NULL) {
+	  ldout(msgr->cct, 20) << "writer no session security" << dendl;
+	} else {
+	  if (session_security->sign_message(m)) {
+	    ldout(msgr->cct, 20) << "writer failed to sign seq # " << header.seq
+				 << "): sig = " << footer.sig << dendl;
+	  } else {
+	    ldout(msgr->cct, 20) << "writer signed seq # " << header.seq
+				 << "): sig = " << footer.sig << dendl;
+	  }
+	}
+
+	bufferlist blist = m->get_payload();
+	blist.append(m->get_middle());
+	blist.append(m->get_data());
+
+	pipe_lock.Unlock();
+
         ldout(msgr->cct,20) << "writer sending " << m->get_seq() << " " << m << dendl;
-	int rc = write_message(m);
+	int rc = write_message(header, footer, blist);
 
 	pipe_lock.Lock();
 	if (rc < 0) {
@@ -1497,8 +1735,19 @@ int Pipe::read_message(Message **pm)
   }
 
   // footer
-  if (tcp_read((char*)&footer, sizeof(footer)) < 0)
-    goto out_dethrottle;
+  if (connection_state->has_feature(CEPH_FEATURE_MSG_AUTH)) {
+    if (tcp_read((char*)&footer, sizeof(footer)) < 0)
+      goto out_dethrottle;
+  } else {
+    ceph_msg_footer_old old_footer;
+    if (tcp_read((char*)&old_footer, sizeof(old_footer)) < 0)
+      goto out_dethrottle;
+    footer.front_crc = old_footer.front_crc;
+    footer.middle_crc = old_footer.middle_crc;
+    footer.data_crc = old_footer.data_crc;
+    footer.sig = 0;
+    footer.flags = old_footer.flags;
+  }
   
   aborted = (footer.flags & CEPH_MSG_FOOTER_COMPLETE) == 0;
   ldout(msgr->cct,10) << "aborted = " << aborted << dendl;
@@ -1517,6 +1766,20 @@ int Pipe::read_message(Message **pm)
     goto out_dethrottle;
   }
 
+  //
+  //  Check the signature if one should be present.  A zero return indicates success. PLR
+  //
+
+  if (session_security == NULL) {
+    ldout(msgr->cct, 10) << "No session security set" << dendl;
+  } else {
+    if (session_security->check_message_signature(message)) {
+      ldout(msgr->cct, 0) << "Signature check failed" << dendl;
+      ret = -EINVAL;
+      goto out_dethrottle;
+    } 
+  }
+
   message->set_throttler(policy.throttler);
 
   // store reservation size in message, so we don't get confused
@@ -1638,25 +1901,10 @@ int Pipe::write_keepalive()
 }
 
 
-int Pipe::write_message(Message *m)
+int Pipe::write_message(ceph_msg_header& header, ceph_msg_footer& footer, bufferlist& blist)
 {
-  ceph_msg_header& header = m->get_header();
-  ceph_msg_footer& footer = m->get_footer();
   int ret;
 
-  // get envelope, buffers
-  header.front_len = m->get_payload().length();
-  header.middle_len = m->get_middle().length();
-  header.data_len = m->get_data().length();
-  footer.flags = CEPH_MSG_FOOTER_COMPLETE;
-  m->calc_header_crc();
-
-  bufferlist blist = m->get_payload();
-  blist.append(m->get_middle());
-  blist.append(m->get_data());
-  
-  ldout(msgr->cct,20)  << "write_message " << m << dendl;
-  
   // set up msghdr and iovecs
   struct msghdr msg;
   memset(&msg, 0, sizeof(msg));
@@ -1739,11 +1987,24 @@ int Pipe::write_message(Message *m)
   }
   assert(left == 0);
 
-  // send footer
-  msgvec[msg.msg_iovlen].iov_base = (void*)&footer;
-  msgvec[msg.msg_iovlen].iov_len = sizeof(footer);
-  msglen += sizeof(footer);
-  msg.msg_iovlen++;
+  // send footer; if receiver doesn't support signatures, use the old footer format
+
+  ceph_msg_footer_old old_footer;
+  if (connection_state->has_feature(CEPH_FEATURE_MSG_AUTH)) {
+    msgvec[msg.msg_iovlen].iov_base = (void*)&footer;
+    msgvec[msg.msg_iovlen].iov_len = sizeof(footer);
+    msglen += sizeof(footer);
+    msg.msg_iovlen++;
+  } else {
+    old_footer.front_crc = footer.front_crc;   
+    old_footer.middle_crc = footer.middle_crc;   
+    old_footer.data_crc = footer.data_crc;   
+    old_footer.flags = footer.flags;   
+    msgvec[msg.msg_iovlen].iov_base = (char*)&old_footer;
+    msgvec[msg.msg_iovlen].iov_len = sizeof(old_footer);
+    msglen += sizeof(old_footer);
+    msg.msg_iovlen++;
+  }
 
   // send
   if (do_sendmsg(&msg, msglen))
