@@ -328,9 +328,6 @@ int HashIndex::_collection_list_partial(const hobject_t &start,
 					vector<hobject_t> *ls,
 					hobject_t *next) {
   vector<string> path;
-  hobject_t _next;
-  if (!next)
-    next = &_next;
   *next = start;
   dout(20) << "_collection_list_partial " << start << " " << min_count << "-" << max_count << " ls.size " << ls->size() << dendl;
   return list_by_hash(path, min_count, max_count, seq, next, ls);
@@ -368,30 +365,21 @@ int HashIndex::start_col_split(const vector<string> &path) {
   bufferlist bl;
   InProgressOp op_tag(InProgressOp::COL_SPLIT, path);
   op_tag.encode(bl);
-  int r = add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl);
-  if (r < 0)
-    return r;
-  return fsync_dir(vector<string>());
+  return add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl); 
 }
 
 int HashIndex::start_split(const vector<string> &path) {
   bufferlist bl;
   InProgressOp op_tag(InProgressOp::SPLIT, path);
   op_tag.encode(bl);
-  int r = add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl);
-  if (r < 0)
-    return r;
-  return fsync_dir(vector<string>());
+  return add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl); 
 }
 
 int HashIndex::start_merge(const vector<string> &path) {
   bufferlist bl;
   InProgressOp op_tag(InProgressOp::MERGE, path);
   op_tag.encode(bl);
-  int r = add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl);
-  if (r < 0)
-    return r;
-  return fsync_dir(vector<string>());
+  return add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl); 
 }
 
 int HashIndex::end_split_or_merge(const vector<string> &path) {
