@@ -41,7 +41,6 @@
 #include "msg/Messenger.h"
 #include "messages/MOSDRepScrub.h"
 #include "messages/MOSDPGLog.h"
-#include "common/WorkQueue.h"
 
 #include "common/DecayCounter.h"
 
@@ -354,7 +353,6 @@ public:
 protected:
   OSDService *osd;
   OSDMapRef osdmap_ref;
-  OSDMapRef last_persisted_osdmap_ref;
   PGPool pool;
 
   OSDMapRef get_osdmap() const {
@@ -817,7 +815,6 @@ public:
       block_writes(false), active(false), queue_snap_trim(false),
       waiting_on(0), errors(0), fixed(0), active_rep_scrub(0),
       must_scrub(false), must_deep_scrub(false), must_repair(false),
-      classic(false),
       finalizing(false), is_chunky(false), state(INACTIVE),
       deep(false)
     {
@@ -853,7 +850,6 @@ public:
     map<hobject_t, pair<ScrubMap::object, int> > authoritative;
 
     // classic scrub
-    bool classic;
     bool finalizing;
 
     // chunky scrub
@@ -925,7 +921,6 @@ public:
 
     // clear all state
     void reset() {
-      classic = false;
       finalizing = false;
       block_writes = false;
       active = false;
@@ -974,28 +969,23 @@ public:
 			  map<hobject_t, int> &authoritative,
 			  map<hobject_t, set<int> > &inconsistent_snapcolls,
 			  ostream &errorstream);
-  void scrub(ThreadPool::TPHandle &handle);
-  void classic_scrub(ThreadPool::TPHandle &handle);
-  void chunky_scrub(ThreadPool::TPHandle &handle);
+  void scrub();
+  void classic_scrub();
+  void chunky_scrub();
   void scrub_compare_maps();
   void scrub_process_inconsistent();
   void scrub_finalize();
   void scrub_finish();
   void scrub_clear_state();
   bool scrub_gather_replica_maps();
-  void _scan_list(
-    ScrubMap &map, vector<hobject_t> &ls, bool deep,
-    ThreadPool::TPHandle &handle);
+  void _scan_list(ScrubMap &map, vector<hobject_t> &ls, bool deep);
   void _request_scrub_map_classic(int replica, eversion_t version);
   void _request_scrub_map(int replica, eversion_t version,
                           hobject_t start, hobject_t end, bool deep);
-  int build_scrub_map_chunk(
-    ScrubMap &map,
-    hobject_t start, hobject_t end, bool deep,
-    ThreadPool::TPHandle &handle);
-  void build_scrub_map(ScrubMap &map, ThreadPool::TPHandle &handle);
-  void build_inc_scrub_map(
-    ScrubMap &map, eversion_t v, ThreadPool::TPHandle &handle);
+  int build_scrub_map_chunk(ScrubMap &map,
+                            hobject_t start, hobject_t end, bool deep);
+  void build_scrub_map(ScrubMap &map);
+  void build_inc_scrub_map(ScrubMap &map, eversion_t v);
   virtual void _scrub(ScrubMap &map) { }
   virtual void _scrub_clear_state() { }
   virtual void _scrub_finish() { }
@@ -1020,9 +1010,7 @@ public:
   void reg_next_scrub();
   void unreg_next_scrub();
 
-  void replica_scrub(
-    class MOSDRepScrub *op,
-    ThreadPool::TPHandle &handle);
+  void replica_scrub(class MOSDRepScrub *op);
   void sub_op_scrub_map(OpRequestRef op);
   void sub_op_scrub_reserve(OpRequestRef op);
   void sub_op_scrub_reserve_reply(OpRequestRef op);
@@ -1351,10 +1339,8 @@ public:
       typedef boost::mpl::list <
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::custom_reaction< MNotifyRec >,
-	boost::statechart::transition< NeedActingChange, WaitActingChange >,
-	boost::statechart::custom_reaction< AdvMap>
+	boost::statechart::transition< NeedActingChange, WaitActingChange >
 	> reactions;
-      boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const ActMap&);
       boost::statechart::result react(const MNotifyRec&);
     };
@@ -1788,13 +1774,18 @@ public:
   void add_log_entry(pg_log_entry_t& e, bufferlist& log_bl);
   void append_log(vector<pg_log_entry_t>& logv, eversion_t trim_to, ObjectStore::Transaction &t);
 
-  void read_log(ObjectStore *store);
+  static void read_log(ObjectStore *store, coll_t coll, hobject_t log_oid,
+    const pg_info_t &info, OndiskLog &ondisklog, IndexedLog &log,
+    pg_missing_t &missing, ostringstream &oss, const PG *passedpg = NULL);
   bool check_log_for_corruption(ObjectStore *store);
   void trim(ObjectStore::Transaction& t, eversion_t v);
   void trim_ondisklog(ObjectStore::Transaction& t);
   void trim_peers();
 
   std::string get_corrupt_pg_log_name() const;
+  static int read_info(ObjectStore *store, const coll_t coll,
+    bufferlist &bl, pg_info_t &info, map<epoch_t,pg_interval_t> &past_intervals,
+    hobject_t &biginfo_oid, interval_set<snapid_t>  &snap_collections);
   void read_state(ObjectStore *store, bufferlist &bl);
   static epoch_t peek_map_epoch(ObjectStore *store,
 				coll_t coll, bufferlist *bl);
@@ -1844,7 +1835,7 @@ public:
   bool old_peering_evt(CephPeeringEvtRef evt) {
     return old_peering_msg(evt->get_epoch_sent(), evt->get_epoch_requested());
   }
-  bool have_same_or_newer_map(epoch_t e) {
+  bool require_same_or_newer_map(epoch_t e) {
     return e <= get_osdmap()->get_epoch();
   }
 
