@@ -62,7 +62,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::split(BufferHead *left, loff_t o
       ++start_remove;
     for (map<loff_t, list<Context*> >::iterator p = start_remove;
 	 p != left->waitfor_read.end(); ++p) {
-      ldout(oc->cct, 20) << "split  moving waiters at byte " << p->first << " to right bh" << dendl;
+      ldout(oc->cct, 0) << "split  moving waiters at byte " << p->first << " to right bh" << dendl;
       right->waitfor_read[p->first].swap( p->second );
       assert(p->second.empty());
     }
@@ -493,26 +493,22 @@ ObjectCacher::ObjectCacher(CephContext *cct_, string name, WritebackHandler& wb,
 			   flush_set_callback_t flush_callback,
 			   void *flush_callback_arg,
 			   uint64_t max_bytes, uint64_t max_objects,
-			   uint64_t max_dirty, uint64_t target_dirty,
-			   double max_dirty_age, bool block_writes_upfront)
+			   uint64_t max_dirty, uint64_t target_dirty, double max_dirty_age)
   : perfcounter(NULL),
     cct(cct_), writeback_handler(wb), name(name), lock(l),
     max_dirty(max_dirty), target_dirty(target_dirty),
     max_size(max_bytes), max_objects(max_objects),
-    block_writes_upfront(block_writes_upfront),
     flush_set_callback(flush_callback), flush_set_callback_arg(flush_callback_arg),
-    flusher_stop(false), flusher_thread(this), finisher(cct),
+    flusher_stop(false), flusher_thread(this),
     stat_clean(0), stat_zero(0), stat_dirty(0), stat_rx(0), stat_tx(0), stat_missing(0),
     stat_error(0), stat_dirty_waiting(0)
 {
   this->max_dirty_age.set_from_double(max_dirty_age);
   perf_start();
-  finisher.start();
 }
 
 ObjectCacher::~ObjectCacher()
 {
-  finisher.stop();
   perf_stop();
   // we should be empty.
   for (vector<hash_map<sobject_t, Object *> >::iterator i = objects.begin();
@@ -598,8 +594,8 @@ void ObjectCacher::bh_read(BufferHead *bh)
   mark_rx(bh);
 
   // finisher
-  C_ReadFinish *onfinish = new C_ReadFinish(this, bh->ob,
-					    bh->start(), bh->length());
+  C_ReadFinish *onfinish = new C_ReadFinish(this, bh->ob->oloc.pool,
+                                            bh->ob->get_soid(), bh->start(), bh->length());
 
   ObjectSet *oset = bh->ob->oset;
 
@@ -611,8 +607,7 @@ void ObjectCacher::bh_read(BufferHead *bh)
 }
 
 void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
-				  uint64_t length, bufferlist &bl, int r,
-				  bool trust_enoent)
+				  uint64_t length, bufferlist &bl, int r)
 {
   assert(lock.is_locked());
   ldout(cct, 7) << "bh_read_finish " 
@@ -639,13 +634,9 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
     Object *ob = objects[poolid][oid];
     
     if (r == -ENOENT && !ob->complete) {
-      // just pass through and retry all waiters if we don't trust
-      // -ENOENT for this read
-      if (trust_enoent) {
-	ldout(cct, 7) << "bh_read_finish ENOENT, marking complete and !exists on " << *ob << dendl;
-	ob->complete = true;
-	ob->exists = false;
-      }
+      ldout(cct, 7) << "bh_read_finish ENOENT, marking complete and !exists on " << *ob << dendl;
+      ob->complete = true;
+      ob->exists = false;
 
       // wake up *all* rx waiters, or else we risk reordering identical reads. e.g.
       //   read 1~1
@@ -710,14 +701,9 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
       opos = bh->end();
 
       if (r == -ENOENT) {
-	if (trust_enoent) {
-	  ldout(cct, 10) << "bh_read_finish removing " << *bh << dendl;
-	  bh_remove(ob, bh);
-	  delete bh;
-	} else {
-	  ldout(cct, 10) << "skipping unstrusted -ENOENT and will retry for "
-			 << *bh << dendl;
-	}
+	ldout(cct, 10) << "bh_read_finish removing " << *bh << dendl;
+	bh_remove(ob, bh);
+	delete bh;
 	continue;
       }
 
@@ -1211,8 +1197,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
 }
 
 
-int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock,
-			 Context *onfreespace)
+int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock)
 {
   assert(lock.is_locked());
   utime_t now = ceph_clock_now(cct);
@@ -1279,83 +1264,53 @@ int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock,
     }
   }
 
-  int r = _wait_for_write(wr, bytes_written, oset, wait_on_lock, onfreespace);
+  int r = _wait_for_write(wr, bytes_written, oset, wait_on_lock);
+
   delete wr;
 
   //verify_stats();
   trim();
   return r;
 }
-
-void ObjectCacher::C_WaitForWrite::finish(int r)
-{
-  Mutex::Locker l(m_oc->lock);
-  m_oc->maybe_wait_for_writeback(m_len);
-  m_onfinish->complete(r);
-}
-
-void ObjectCacher::maybe_wait_for_writeback(uint64_t len)
-{
-  assert(lock.is_locked());
-  utime_t start = ceph_clock_now(cct);
-  int blocked = 0;
-  // wait for writeback?
-  //  - wait for dirty and tx bytes (relative to the max_dirty threshold)
-  //  - do not wait for bytes other waiters are waiting on.  this means that
-  //    threads do not wait for each other.  this effectively allows the cache
-  //    size to balloon proportional to the data that is in flight.
-  while (get_stat_dirty() + get_stat_tx() >= max_dirty + get_stat_dirty_waiting()) {
-    ldout(cct, 10) << __func__ << " waiting for dirty|tx "
-		   << (get_stat_dirty() + get_stat_tx()) << " >= max "
-		   << max_dirty << " + dirty_waiting "
-		   << get_stat_dirty_waiting() << dendl;
-    flusher_cond.Signal();
-    stat_dirty_waiting += len;
-    stat_cond.Wait(lock);
-    stat_dirty_waiting -= len;
-    ++blocked;
-    ldout(cct, 10) << __func__ << " woke up" << dendl;
-  }
-  if (blocked && perfcounter) {
-    perfcounter->inc(l_objectcacher_write_ops_blocked);
-    perfcounter->inc(l_objectcacher_write_bytes_blocked, len);
-    utime_t blocked = ceph_clock_now(cct) - start;
-    perfcounter->tinc(l_objectcacher_write_time_blocked, blocked);
-  }
-}
+ 
 
 // blocking wait for write.
-int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset, Mutex& lock, Context *onfreespace)
+int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset, Mutex& lock)
 {
   assert(lock.is_locked());
+  int blocked = 0;
+  utime_t start = ceph_clock_now(cct);
   int ret = 0;
 
   if (max_dirty > 0) {
-    if (block_writes_upfront) {
-      maybe_wait_for_writeback(len);
-      if (onfreespace)
-	onfreespace->complete(0);
-    } else {
-      assert(onfreespace);
-      finisher.queue(new C_WaitForWrite(this, len, onfreespace));
+    // wait for writeback?
+    //  - wait for dirty and tx bytes (relative to the max_dirty threshold)
+    //  - do not wait for bytes other waiters are waiting on.  this means that
+    //    threads do not wait for each other.  this effectively allows the cache size
+    //    to balloon proportional to the data that is in flight.
+    while (get_stat_dirty() + get_stat_tx() >= max_dirty + get_stat_dirty_waiting()) {
+      ldout(cct, 10) << "wait_for_write waiting on " << len << ", dirty|tx " 
+		     << (get_stat_dirty() + get_stat_tx()) 
+		     << " >= max " << max_dirty << " + dirty_waiting " << get_stat_dirty_waiting()
+		     << dendl;
+      flusher_cond.Signal();
+      stat_dirty_waiting += len;
+      stat_cond.Wait(lock);
+      stat_dirty_waiting -= len;
+      blocked++;
+      ldout(cct, 10) << "wait_for_write woke up" << dendl;
     }
   } else {
     // write-thru!  flush what we just wrote.
     Cond cond;
     bool done;
-    Context *fin = block_writes_upfront ?
-      new C_Cond(&cond, &done, &ret) : onfreespace;
-    assert(fin);
+    C_Cond *fin = new C_Cond(&cond, &done, &ret);
     bool flushed = flush_set(oset, wr->extents, fin);
     assert(!flushed);   // we just dirtied it, and didn't drop our lock!
     ldout(cct, 10) << "wait_for_write waiting on write-thru of " << len << " bytes" << dendl;
-    if (block_writes_upfront) {
-      while (!done)
-	cond.Wait(lock);
-      ldout(cct, 10) << "wait_for_write woke up, ret " << ret << dendl;
-      if (onfreespace)
-	onfreespace->complete(ret);
-    }
+    while (!done)
+      cond.Wait(lock);
+    ldout(cct, 10) << "wait_for_write woke up, ret " << ret << dendl;
   }
 
   // start writeback anyway?
@@ -1364,6 +1319,12 @@ int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset, M
 		   << target_dirty << ", nudging flusher" << dendl;
     flusher_cond.Signal();
   }
+  if (blocked && perfcounter) {
+    perfcounter->inc(l_objectcacher_write_ops_blocked);
+    perfcounter->inc(l_objectcacher_write_bytes_blocked, len);
+    utime_t blocked = ceph_clock_now(cct) - start;
+    perfcounter->tinc(l_objectcacher_write_time_blocked, blocked);
+  }
   return ret;
 }
 
@@ -1496,52 +1457,47 @@ bool ObjectCacher::flush(Object *ob, loff_t offset, loff_t length)
   return clean;
 }
 
-bool ObjectCacher::_flush_set_finish(C_GatherBuilder *gather, Context *onfinish)
-{
-  assert(lock.is_locked());
-  if (gather->has_subs()) {
-    gather->set_finisher(onfinish);
-    gather->activate();
-    return false;
-  }
-
-  ldout(cct, 10) << "flush_set has no dirty|tx bhs" << dendl;
-  onfinish->complete(0);
-  return true;
-}
-
 // flush.  non-blocking, takes callback.
 // returns true if already flushed
 bool ObjectCacher::flush_set(ObjectSet *oset, Context *onfinish)
 {
   assert(lock.is_locked());
-  assert(onfinish != NULL);
   if (oset->objects.empty()) {
     ldout(cct, 10) << "flush_set on " << oset << " dne" << dendl;
-    onfinish->complete(0);
+    delete onfinish;
     return true;
   }
 
   ldout(cct, 10) << "flush_set " << oset << dendl;
 
   // we'll need to wait for all objects to flush!
-  C_GatherBuilder gather(cct);
+  C_GatherBuilder gather(cct, onfinish);
 
+  bool safe = true;
   for (xlist<Object*>::iterator i = oset->objects.begin();
        !i.end(); ++i) {
     Object *ob = *i;
 
     if (!flush(ob, 0, 0)) {
       // we'll need to gather...
+      safe = false;
+
       ldout(cct, 10) << "flush_set " << oset << " will wait for ack tid " 
                << ob->last_write_tid 
                << " on " << *ob
                << dendl;
-      ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
+      if (onfinish != NULL)
+        ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
     }
   }
-
-  return _flush_set_finish(&gather, onfinish);
+  if (onfinish != NULL)
+    gather.activate();
+  
+  if (safe) {
+    ldout(cct, 10) << "flush_set " << oset << " has no dirty|tx bhs" << dendl;
+    return true;
+  }
+  return false;
 }
 
 // flush.  non-blocking, takes callback.
@@ -1549,19 +1505,18 @@ bool ObjectCacher::flush_set(ObjectSet *oset, Context *onfinish)
 bool ObjectCacher::flush_set(ObjectSet *oset, vector<ObjectExtent>& exv, Context *onfinish)
 {
   assert(lock.is_locked());
-  assert(onfinish != NULL);
   if (oset->objects.empty()) {
     ldout(cct, 10) << "flush_set on " << oset << " dne" << dendl;
-    onfinish->complete(0);
+    delete onfinish;
     return true;
   }
 
-  ldout(cct, 10) << "flush_set " << oset << " on " << exv.size()
-		 << " ObjectExtents" << dendl;
+  ldout(cct, 10) << "flush_set " << oset << " on " << exv.size() << " ObjectExtents" << dendl;
 
   // we'll need to wait for all objects to flush!
-  C_GatherBuilder gather(cct);
+  C_GatherBuilder gather(cct, onfinish);
 
+  bool safe = true;
   for (vector<ObjectExtent>::iterator p = exv.begin();
        p != exv.end();
        ++p) {
@@ -1575,13 +1530,22 @@ bool ObjectCacher::flush_set(ObjectSet *oset, vector<ObjectExtent>& exv, Context
 
     if (!flush(ob, ex.offset, ex.length)) {
       // we'll need to gather...
+      safe = false;
+
       ldout(cct, 10) << "flush_set " << oset << " will wait for ack tid " 
 		     << ob->last_write_tid << " on " << *ob << dendl;
-      ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
+      if (onfinish != NULL)
+        ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
     }
   }
-
-  return _flush_set_finish(&gather, onfinish);
+  if (onfinish != NULL)
+    gather.activate();
+  
+  if (safe) {
+    ldout(cct, 10) << "flush_set " << oset << " has no dirty|tx bhs" << dendl;
+    return true;
+  }
+  return false;
 }
 
 void ObjectCacher::purge_set(ObjectSet *oset)
@@ -1717,26 +1681,7 @@ uint64_t ObjectCacher::release_all()
   return unclean;
 }
 
-void ObjectCacher::clear_nonexistence(ObjectSet *oset)
-{
-  assert(lock.is_locked());
-  ldout(cct, 10) << "clear_nonexistence() " << oset << dendl;
 
-  for (xlist<Object*>::iterator p = oset->objects.begin();
-       !p.end(); ++p) {
-    Object *ob = *p;
-    if (!ob->exists) {
-      ldout(cct, 10) << " setting exists and complete on " << *ob << dendl;
-      ob->exists = true;
-      ob->complete = false;
-    }
-    for (xlist<C_ReadFinish*>::iterator q = ob->reads.begin();
-	 !q.end(); ++q) {
-      C_ReadFinish *comp = *q;
-      comp->distrust_enoent();
-    }
-  }
-}
 
 /**
  * discard object extents from an ObjectSet by removing the objects in exls from the in-memory oset.
