@@ -19,6 +19,7 @@
 #include "auth/KeyRing.h"
 #include "common/errno.h"
 #include "common/ceph_argparse.h"
+#include "common/strtol.h"
 #include "global/global_init.h"
 #include "common/safe_io.h"
 #include "common/secret.h"
@@ -32,6 +33,7 @@
 #include "include/compat.h"
 #include "common/blkdev.h"
 
+#include <boost/scoped_ptr.hpp>
 #include <dirent.h>
 #include <errno.h>
 #include <inttypes.h>
@@ -48,6 +50,8 @@
 #include "common/TextTable.h"
 #include "include/util.h"
 
+#include "common/Formatter.h"
+
 #if defined(__linux__)
 #include <linux/fs.h>
 #endif
@@ -64,6 +68,8 @@
 static string dir_oid = RBD_DIRECTORY;
 static string dir_info_oid = RBD_INFO;
 
+bool udevadm_settle = true;
+
 void usage()
 {
   cout << 
@@ -113,37 +119,62 @@ void usage()
 "individual pieces of names with -p/--pool, --image, and/or --snap.\n"
 "\n"
 "Other input options:\n"
-"  -p, --pool <pool>            source pool name\n"
-"  --image <image-name>         image name\n"
-"  --dest <image-name>          destination [pool and] image name\n"
-"  --snap <snap-name>           snapshot name\n"
-"  --dest-pool <name>           destination pool name\n"
-"  --path <path-name>           path name for import/export\n"
-"  --size <size in MB>          size of image for create and resize\n"
-"  --order <bits>               the object size in bits; object size will be\n"
-"                               (1 << order) bytes. Default is 22 (4 MB).\n"
-"  --format <format-number>     format to use when creating an image\n"
-"                               format 1 is the original format (default)\n"
-"                               format 2 supports cloning\n"
-"  --id <username>              rados user (without 'client.' prefix) to authenticate as\n"
-"  --keyfile <path>             file containing secret key for use with cephx\n"
-"  --shared <tag>               take a shared (rather than exclusive) lock\n";
+"  -p, --pool <pool>                  source pool name\n"
+"  --image <image-name>               image name\n"
+"  --dest <image-name>                destination [pool and] image name\n"
+"  --snap <snap-name>                 snapshot name\n"
+"  --dest-pool <name>                 destination pool name\n"
+"  --path <path-name>                 path name for import/export\n"
+"  --size <size in MB>                size of image for create and resize\n"
+"  --order <bits>                     the object size in bits; object size will be\n"
+"                                     (1 << order) bytes. Default is 22 (4 MB).\n"
+"  --image-format <format-number>     format to use when creating an image\n"
+"                                     format 1 is the original format (default)\n"
+"                                     format 2 supports cloning\n"
+"  --id <username>                    rados user (without 'client.'prefix) to\n"
+"                                     authenticate as\n"
+"  --keyfile <path>                   file containing secret key for use with cephx\n"
+"  --shared <tag>                     take a shared (rather than exclusive) lock\n"
+"  --format <output-format>           output format (default: plain, json, xml)\n"
+"  --pretty-format                    make json or xml output more readable\n"
+"  --no-settle                        do not wait for udevadm to settle on map/unmap\n";
 }
 
-static string feature_str(uint64_t features)
+static string feature_str(uint64_t feature)
 {
-  string s = ""; 
+  switch (feature) {
+  case RBD_FEATURE_LAYERING:
+    return "layering";
+  case RBD_FEATURE_STRIPINGV2:
+    return "striping";
+  default:
+    return "";
+  }
+}
 
-  if (features & RBD_FEATURE_LAYERING)
-    s += "layering";
-  if (features & RBD_FEATURE_STRIPINGV2) {
+static string features_str(uint64_t features)
+{
+  string s = "";
+
+  for (uint64_t feature = 1; feature <= RBD_FEATURE_STRIPINGV2;
+       feature <<= 1) {
     if (s.size())
       s += ", ";
-    s += "striping";
+    s += feature_str(feature);
   }
   return s;
 }
 
+static void format_features(Formatter *f, uint64_t features)
+{
+  f->open_array_section("features");
+  for (uint64_t feature = 1; feature <= RBD_FEATURE_STRIPINGV2;
+       feature <<= 1) {
+    f->dump_string("feature", feature_str(feature));
+  }
+  f->close_section();
+}
+
 struct MyProgressContext : public librbd::ProgressContext {
   const char *operation;
   int last_pc;
@@ -166,32 +197,64 @@ struct MyProgressContext : public librbd::ProgressContext {
     cout << "\r" << operation << ": 100% complete...done." << std::endl;
   }
   void fail() {
-    cout << "\r" << operation << ": " << last_pc << "% complete...failed." << std::endl;
+    cout << "\r" << operation << ": " << last_pc << "% complete...failed."
+	 << std::endl;
   }
 };
 
-static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag)
+static int get_outfmt(const char *output_format,
+		      bool pretty,
+		      boost::scoped_ptr<Formatter> *f)
+{
+  if (!strcmp(output_format, "json")) {
+    f->reset(new JSONFormatter(pretty));
+  } else if (!strcmp(output_format, "xml")) {
+    f->reset(new XMLFormatter(pretty));
+  } else if (strcmp(output_format, "plain")) {
+    cerr << "rbd: unknown format '" << output_format << "'" << std::endl;
+    return -EINVAL;
+  }
+
+  return 0;
+}
+
+static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag,
+		   Formatter *f)
 {
   std::vector<string> names;
   int r = rbd.list(io_ctx, names);
-  if (r < 0 || (names.size() == 0))
+  if (r < 0)
     return r;
 
   if (!lflag) {
+    if (f)
+      f->open_array_section("images");
     for (std::vector<string>::const_iterator i = names.begin();
        i != names.end(); ++i) {
-      cout << *i << std::endl;
+       if (f)
+	 f->dump_string("name", *i);
+       else
+	 cout << *i << std::endl;
+    }
+    if (f) {
+      f->close_section();
+      f->flush(cout);
     }
     return 0;
   }
 
   TextTable tbl;
-  tbl.define_column("NAME", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("SIZE", TextTable::RIGHT, TextTable::RIGHT);
-  tbl.define_column("PARENT", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("FMT", TextTable::RIGHT, TextTable::RIGHT);
-  tbl.define_column("PROT", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("LOCK", TextTable::LEFT, TextTable::LEFT);
+
+  if (f) {
+    f->open_array_section("images");
+  } else {
+    tbl.define_column("NAME", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("SIZE", TextTable::RIGHT, TextTable::RIGHT);
+    tbl.define_column("PARENT", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("FMT", TextTable::RIGHT, TextTable::RIGHT);
+    tbl.define_column("PROT", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("LOCK", TextTable::LEFT, TextTable::LEFT);
+  }
 
   string pool, image, snap, parent;
 
@@ -220,8 +283,11 @@ static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag)
     if (r < 0 && r != -ENOENT)
       return r;
 
-    if (r != -ENOENT)
+    bool has_parent = false;
+    if (r != -ENOENT) {
       parent = pool + "/" + image + "@" + snap;
+      has_parent = true;
+    }
 
     if (im.stat(info, sizeof(info)) < 0)
       return -EINVAL;
@@ -239,38 +305,80 @@ static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag)
       lockstr = (exclusive) ? "excl" : "shr";
     }
 
-    tbl << *i
-	<< stringify(si_t(info.size))
-	<< parent
-	<< ((old_format) ? '1' : '2')
-	<< ""				// protect doesn't apply to images
-	<< lockstr
-	<< TextTable::endrow;
+    if (f) {
+      f->open_object_section("image");
+      f->dump_string("image", *i);
+      f->dump_unsigned("size", info.size);
+      if (has_parent) {
+	f->open_object_section("parent");
+	f->dump_string("pool", pool);
+	f->dump_string("image", image);
+	f->dump_string("snapshot", snap);
+	f->close_section();
+      }
+      f->dump_int("format", old_format ? 1 : 2);
+      if (lockers.size())
+	f->dump_string("lock_type", exclusive ? "exclusive" : "shared");
+      f->close_section();
+    } else {
+      tbl << *i
+	  << stringify(si_t(info.size))
+	  << parent
+	  << ((old_format) ? '1' : '2')
+	  << ""				// protect doesn't apply to images
+	  << lockstr
+	  << TextTable::endrow;
+    }
 
     vector<librbd::snap_info_t> snaplist;
     if (im.snap_list(snaplist) >= 0 && !snaplist.empty()) {
       for (std::vector<librbd::snap_info_t>::iterator s = snaplist.begin();
-           s != snaplist.end(); ++s) {
+	   s != snaplist.end(); ++s) {
 	bool is_protected;
+	bool has_parent = false;
 	parent.clear();
 	im.snap_set(s->name.c_str());
-        r = im.snap_is_protected(s->name.c_str(), &is_protected);
+	r = im.snap_is_protected(s->name.c_str(), &is_protected);
 	if (r < 0)
 	  return r;
 	if (im.parent_info(&pool, &image, &snap) >= 0) {
 	  parent = pool + "/" + image + "@" + snap;
+	  has_parent = true;
+	}
+	if (f) {
+	  f->open_object_section("snapshot");
+	  f->dump_string("image", *i);
+	  f->dump_string("snapshot", s->name);
+	  f->dump_unsigned("size", s->size);
+	  if (has_parent) {
+	    f->open_object_section("parent");
+	    f->dump_string("pool", pool);
+	    f->dump_string("image", image);
+	    f->dump_string("snapshot", snap);
+	    f->close_section();
+	  }
+	  f->dump_int("format", old_format ? 1 : 2);
+	  f->dump_string("protected", is_protected ? "true" : "false");
+	  f->close_section();
+	} else {
+	  tbl << *i + "@" + s->name
+	      << stringify(si_t(s->size))
+	      << parent
+	      << ((old_format) ? '1' : '2')
+	      << (is_protected ? "yes" : "")
+	      << "" 			// locks don't apply to snaps
+	      << TextTable::endrow;
 	}
-        tbl << *i + "@" + s->name
-	    << stringify(si_t(s->size))
-	    << parent
-	    << ((old_format) ? '1' : '2')
-	    << (is_protected ? "yes" : "")
-	    << "" 			// locks don't apply to snaps
-	    << TextTable::endrow;
       }
     }
   }
-  cout << tbl;
+  if (f) {
+    f->close_section();
+    f->flush(cout);
+  } else if (names.size()) {
+    cout << tbl;
+  }
+
   return 0;
 }
 
@@ -285,7 +393,8 @@ static int do_create(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     // weird striping not allowed with format 1!
     if ((stripe_unit || stripe_count) &&
 	(stripe_unit != (1ull << *order) && stripe_count != 1)) {
-      cerr << "non-default striping not allowed with format 1; use --format 2" << std::endl;
+      cerr << "non-default striping not allowed with format 1; use --format 2"
+	   << std::endl;
       return -EINVAL;
     }
     r = rbd.create(io_ctx, imgname, size, order);
@@ -295,7 +404,8 @@ static int do_create(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     }
     if (stripe_unit != (1ull << *order) && stripe_count != 1)
       features |= RBD_FEATURE_STRIPINGV2;
-    r = rbd.create3(io_ctx, imgname, size, features, order, stripe_unit, stripe_count);
+    r = rbd.create3(io_ctx, imgname, size, features, order,
+		    stripe_unit, stripe_count);
   }
   if (r < 0)
     return r;
@@ -338,15 +448,16 @@ static int do_rename(librbd::RBD &rbd, librados::IoCtx& io_ctx,
 }
 
 static int do_show_info(const char *imgname, librbd::Image& image,
-			const char *snapname)
+			const char *snapname, Formatter *f)
 {
   librbd::image_info_t info;
   string parent_pool, parent_name, parent_snapname;
   uint8_t old_format;
   uint64_t overlap, features;
   bool snap_protected;
+  int r;
 
-  int r = image.stat(info, sizeof(info));
+  r = image.stat(info, sizeof(info));
   if (r < 0)
     return r;
 
@@ -368,45 +479,85 @@ static int do_show_info(const char *imgname, librbd::Image& image,
       return r;
   }
 
-  cout << "rbd image '" << imgname << "':\n"
-       << "\tsize " << prettybyte_t(info.size) << " in "
-       << info.num_objs << " objects"
-       << std::endl
-       << "\torder " << info.order
-       << " (" << prettybyte_t(info.obj_size) << " objects)"
-       << std::endl
-       << "\tblock_name_prefix: " << info.block_name_prefix
-       << std::endl
-       << "\tformat: " << (old_format ? "1" : "2")
-       << std::endl;
+  if (f) {
+    f->open_object_section("image");
+    f->dump_string("name", imgname);
+    f->dump_unsigned("size", info.size);
+    f->dump_unsigned("objects", info.num_objs);
+    f->dump_int("order", info.order);
+    f->dump_unsigned("object_size", info.obj_size);
+    f->dump_string("block_name_prefix", info.block_name_prefix);
+    f->dump_int("format", (old_format ? 1 : 2));
+  } else {
+    cout << "rbd image '" << imgname << "':\n"
+	 << "\tsize " << prettybyte_t(info.size) << " in "
+	 << info.num_objs << " objects"
+	 << std::endl
+	 << "\torder " << info.order
+	 << " (" << prettybyte_t(info.obj_size) << " objects)"
+	 << std::endl
+	 << "\tblock_name_prefix: " << info.block_name_prefix
+	 << std::endl
+	 << "\tformat: " << (old_format ? "1" : "2")
+	 << std::endl;
+  }
+
   if (!old_format) {
-    cout << "\tfeatures: " << feature_str(features) << std::endl;
+    if (f)
+      format_features(f, features);
+    else
+      cout << "\tfeatures: " << features_str(features) << std::endl;
   }
 
   // snapshot info, if present
   if (snapname) {
-    cout << "\tprotected: " << (snap_protected ? "True" : "False")
-	 << std::endl;
+    if (f) {
+      f->dump_string("protected", snap_protected ? "true" : "false");
+    } else {
+      cout << "\tprotected: " << (snap_protected ? "True" : "False")
+	   << std::endl;
+    }
   }
 
   // parent info, if present
   if ((image.parent_info(&parent_pool, &parent_name, &parent_snapname) == 0) &&
       parent_name.length() > 0) {
-
-    cout << "\tparent: " << parent_pool << "/" << parent_name
-	 << "@" << parent_snapname << std::endl;
-    cout << "\toverlap: " << prettybyte_t(overlap) << std::endl;
+    if (f) {
+      f->open_object_section("parent");
+      f->dump_string("pool", parent_pool);
+      f->dump_string("image", parent_name);
+      f->dump_string("snapshot", parent_snapname);
+      f->dump_unsigned("overlap", overlap);
+      f->close_section();
+    } else {
+      cout << "\tparent: " << parent_pool << "/" << parent_name
+	   << "@" << parent_snapname << std::endl;
+      cout << "\toverlap: " << prettybyte_t(overlap) << std::endl;
+    }
   }
 
   // striping info, if feature is set
   if (features & RBD_FEATURE_STRIPINGV2) {
-    cout << "\tstripe unit: " << prettybyte_t(image.get_stripe_unit()) << std::endl
-	 << "\tstripe count: " << image.get_stripe_count() << std::endl;
+    if (f) {
+      f->dump_unsigned("stripe_unit", image.get_stripe_unit());
+      f->dump_unsigned("stripe_count", image.get_stripe_count());
+    } else {
+      cout << "\tstripe unit: " << prettybyte_t(image.get_stripe_unit())
+	   << std::endl
+	   << "\tstripe count: " << image.get_stripe_count() << std::endl;
+    }
   }
+
+  if (f) {
+    f->close_section();
+    f->flush(cout);
+  }
+
   return 0;
 }
 
-static int do_delete(librbd::RBD &rbd, librados::IoCtx& io_ctx, const char *imgname)
+static int do_delete(librbd::RBD &rbd, librados::IoCtx& io_ctx,
+		     const char *imgname)
 {
   MyProgressContext pc("Removing image");
   int r = rbd.remove_with_progress(io_ctx, imgname, pc);
@@ -430,24 +581,45 @@ static int do_resize(librbd::Image& image, uint64_t size)
   return 0;
 }
 
-static int do_list_snaps(librbd::Image& image)
+static int do_list_snaps(librbd::Image& image, Formatter *f)
 {
   std::vector<librbd::snap_info_t> snaps;
-  int r = image.snap_list(snaps);
-  if (r < 0 || snaps.empty())
+  TextTable t;
+  int r;
+
+  r = image.snap_list(snaps);
+  if (r < 0)
     return r;
 
-  TextTable t;
-  t.define_column("SNAPID", TextTable::RIGHT, TextTable::RIGHT);
-  t.define_column("NAME", TextTable::LEFT, TextTable::LEFT);
-  t.define_column("SIZE", TextTable::RIGHT, TextTable::RIGHT);
+  if (f) {
+    f->open_array_section("snapshots");
+  } else {
+    t.define_column("SNAPID", TextTable::RIGHT, TextTable::RIGHT);
+    t.define_column("NAME", TextTable::LEFT, TextTable::LEFT);
+    t.define_column("SIZE", TextTable::RIGHT, TextTable::RIGHT);
+  }
 
   for (std::vector<librbd::snap_info_t>::iterator s = snaps.begin();
        s != snaps.end(); ++s) {
-    t << s->id << s->name << stringify(prettybyte_t(s->size))
-      << TextTable::endrow;
+    if (f) {
+      f->open_object_section("snapshot");
+      f->dump_unsigned("id", s->id);
+      f->dump_string("name", s->name);
+      f->dump_unsigned("size", s->size);
+      f->close_section();
+    } else {
+      t << s->id << s->name << stringify(prettybyte_t(s->size))
+	<< TextTable::endrow;
+    }
   }
-  cout << t;
+
+  if (f) {
+    f->close_section();
+    f->flush(cout);
+  } else if (snaps.size()) {
+    cout << t;
+  }
+
   return 0;
 }
 
@@ -522,47 +694,87 @@ static int do_unprotect_snap(librbd::Image& image, const char *snapname)
   return 0;
 }
 
-static int do_list_children(librbd::Image &image)
+static int do_list_children(librbd::Image &image, Formatter *f)
 {
   set<pair<string, string> > children;
-  int r = image.list_children(&children);
+  int r;
+
+  r = image.list_children(&children);
   if (r < 0)
     return r;
 
+  if (f)
+    f->open_array_section("children");
+
   for (set<pair<string, string> >::const_iterator child_it = children.begin();
        child_it != children.end(); child_it++) {
-    cout << child_it->first << "/" << child_it->second << std::endl;
+    if (f) {
+      f->open_object_section("child");
+      f->dump_string("pool", child_it->first);
+      f->dump_string("image", child_it->second);
+      f->close_section();
+    } else {
+      cout << child_it->first << "/" << child_it->second << std::endl;
+    }
   }
+
+  if (f) {
+    f->close_section();
+    f->flush(cout);
+  }
+
   return 0;
 }
 
-static int do_lock_list(librbd::Image& image)
+static int do_lock_list(librbd::Image& image, Formatter *f)
 {
   list<librbd::locker_t> lockers;
   bool exclusive;
   string tag;
-  int r = image.list_lockers(&lockers, &exclusive, &tag);
+  TextTable tbl;
+  int r;
+
+  r = image.list_lockers(&lockers, &exclusive, &tag);
   if (r < 0)
     return r;
 
-  if (lockers.size()) {
-    bool one = (lockers.size() == 1);
-    cout << "There " << (one ? "is " : "are ") << lockers.size()
-	 << (exclusive ? " exclusive" : " shared")
-	 << " lock" << (one ? "" : "s") << " on this image.\n";
-    if (!exclusive)
-      cout << "Lock tag: " << tag << "\n";
-
-    TextTable tbl;
+  if (f) {
+    f->open_object_section("locks");
+  } else {
     tbl.define_column("Locker", TextTable::LEFT, TextTable::LEFT);
     tbl.define_column("ID", TextTable::LEFT, TextTable::LEFT);
     tbl.define_column("Address", TextTable::LEFT, TextTable::LEFT);
+  }
+
+  if (lockers.size()) {
+    bool one = (lockers.size() == 1);
+
+    if (!f) {
+      cout << "There " << (one ? "is " : "are ") << lockers.size()
+	   << (exclusive ? " exclusive" : " shared")
+	   << " lock" << (one ? "" : "s") << " on this image.\n";
+      if (!exclusive)
+	cout << "Lock tag: " << tag << "\n";
+    }
 
     for (list<librbd::locker_t>::const_iterator it = lockers.begin();
 	 it != lockers.end(); ++it) {
-      tbl << it->client << it->cookie << it->address << TextTable::endrow;
+      if (f) {
+	f->open_object_section(it->cookie.c_str());
+	f->dump_string("locker", it->client);
+	f->dump_string("address", it->address);
+	f->close_section();
+      } else {
+	tbl << it->client << it->cookie << it->address << TextTable::endrow;
+      }
     }
-    cout << tbl;
+    if (!f)
+      cout << tbl;
+  }
+
+  if (f) {
+    f->close_section();
+    f->flush(cout);
   }
   return 0;
 }
@@ -598,12 +810,16 @@ struct rbd_bencher {
       in_flight(0)
   { }
 
-  bool start_write(int max, uint64_t off, uint64_t len, bufferlist& bl) {
-    Mutex::Locker l(lock);
-    if (in_flight >= max)
-      return false;
-    in_flight++;
-    librbd::RBD::AioCompletion *c = new librbd::RBD::AioCompletion((void *)this, rbd_bencher_completion);
+  bool start_write(int max, uint64_t off, uint64_t len, bufferlist& bl)
+  {
+    {
+      Mutex::Locker l(lock);
+      if (in_flight >= max)
+	return false;
+      in_flight++;
+    }
+    librbd::RBD::AioCompletion *c =
+      new librbd::RBD::AioCompletion((void *)this, rbd_bencher_completion);
     image->aio_write(off, len, bl, c);
     //cout << "start " << c << " at " << off << "~" << len << std::endl;
     return true;
@@ -632,7 +848,8 @@ void rbd_bencher_completion(void *vc, void *pc)
   c->release();
 }
 
-static int do_bench_write(librbd::Image& image, uint64_t io_size, uint64_t io_threads, uint64_t io_bytes)
+static int do_bench_write(librbd::Image& image, uint64_t io_size,
+			  uint64_t io_threads, uint64_t io_bytes)
 {
   rbd_bencher b(&image);
 
@@ -655,26 +872,35 @@ static int do_bench_write(librbd::Image& image, uint64_t io_size, uint64_t io_th
   uint64_t off;
   for (off = 0; off < io_bytes; off += io_size) {
     b.wait_for(io_threads - 1);
-    while (b.start_write(io_threads, off, io_size, bl))
-      ios++;
+    uint64_t i = 0;
+    while (i < io_threads &&
+	   b.start_write(io_threads, off, io_size, bl)) {
+      ++i;
+      ++ios;
+    }
 
     utime_t now = ceph_clock_now(NULL);
     utime_t elapsed = now - start;
     if (elapsed.sec() != last.sec()) {
-      printf("%5d  %8d  %8.2lf  %8.2lf\n", (int)elapsed, (int)(ios - io_threads),
+      printf("%5d  %8d  %8.2lf  %8.2lf\n",
+	     (int)elapsed,
+	     (int)(ios - io_threads),
 	     (double)(ios - io_threads) / elapsed,
 	     (double)(off - io_threads * io_size) / elapsed);
       last = elapsed;
     }
   }
   b.wait_for(0);
+  int r = image.flush();
+  if (r < 0) {
+    cerr << "Error flushing data at the end: " << cpp_strerror(r) << std::endl;
+  }
 
   utime_t now = ceph_clock_now(NULL);
   double elapsed = now - start;
 
-  printf("elapsed: %5d  ops: %8d  ops/sec: %8.2lf  bytes/sec: %8.2lf\n", (int)elapsed, ios,
-	     (double)ios / elapsed,
-	     (double)off / elapsed);
+  printf("elapsed: %5d  ops: %8d  ops/sec: %8.2lf  bytes/sec: %8.2lf\n",
+	 (int)elapsed, ios, (double)ios / elapsed, (double)off / elapsed);
 
   return 0;
 }
@@ -684,7 +910,8 @@ struct ExportContext {
   uint64_t totalsize;
   MyProgressContext pc;
 
-  ExportContext(int f, uint64_t t) : fd(f), totalsize(t), pc("Exporting image") {}
+  ExportContext(int f, uint64_t t) : fd(f), totalsize(t), pc("Exporting image")
+  {}
 };
 
 static int export_read_cb(uint64_t ofs, size_t len, const char *buf, void *arg)
@@ -888,7 +1115,8 @@ static int do_import_from_stdin(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     if (!bl.is_zero()) {
       r = image.write(image_pos, blklen, bl);
       if (r < 0) {
-	cerr << "rbd: error writing to image block" << cpp_strerror(r) << std::endl;
+	cerr << "rbd: error writing to image block" << cpp_strerror(r)
+	     << std::endl;
 	return r;
       }
     }
@@ -968,7 +1196,8 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     return r;
   }
 
-  fsync(fd); /* flush it first, otherwise extents information might not have been flushed yet */
+  // flush it first, otherwise extents may not be allocated or readable
+  fsync(fd);
   fiemap = read_fiemap(fd);
   if (fiemap && !fiemap->fm_mapped_extents) {
     cerr << "rbd: empty fiemap!" << std::endl;
@@ -976,7 +1205,8 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     fiemap = NULL;
   }
   if (!fiemap) {
-    fiemap = (struct fiemap *)malloc(sizeof(struct fiemap) +  sizeof(struct fiemap_extent));
+    size_t fiemap_size = sizeof(struct fiemap) + sizeof(struct fiemap_extent);
+    fiemap = (struct fiemap *)malloc(fiemap_size);
     if (!fiemap) {
       cerr << "rbd: failed to allocate fiemap, not enough memory." << std::endl;
       close(fd);
@@ -1000,7 +1230,8 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     size_t extent_len = 0;
     size_t read_len = 1ULL << *order;
 
-    file_pos = fiemap->fm_extents[extent].fe_logical; /* position within the file we're reading */
+    /* position within the file we're reading */
+    file_pos = fiemap->fm_extents[extent].fe_logical;
 
     do { /* try to merge consecutive extents */
 #define LARGE_ENOUGH_EXTENT (32 * 1024 * 1024)
@@ -1008,7 +1239,8 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
           extent_len + fiemap->fm_extents[extent].fe_length > LARGE_ENOUGH_EXTENT)
         break; /* don't try to merge if we're big enough */
 
-      extent_len += fiemap->fm_extents[extent].fe_length;  /* length of current extent */
+      /* length of current extent */
+      extent_len += fiemap->fm_extents[extent].fe_length;
       end_ofs = MIN((off_t)size, file_pos + (off_t)extent_len);
 
       extent++;
@@ -1041,7 +1273,8 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
         }
         bufferlist bl;
         bl.append(p);
-        librbd::RBD::AioCompletion *completion = new librbd::RBD::AioCompletion(NULL, NULL);
+        librbd::RBD::AioCompletion *completion =
+	  new librbd::RBD::AioCompletion(NULL, NULL);
         if (!completion) {
           r = -ENOMEM;
           goto done;
@@ -1082,9 +1315,11 @@ static int do_copy(librbd::Image &src, librados::IoCtx& dest_pp,
 {
   MyProgressContext pc("Image copy");
   int r = src.copy_with_progress(dest_pp, destname, pc);
-  pc.finish();
-  if (r < 0)
+  if (r < 0){
+    pc.fail();
     return r;
+  }
+  pc.finish();
   return 0;
 }
 
@@ -1094,7 +1329,8 @@ public:
   RbdWatchCtx(const char *imgname) : name(imgname) {}
   virtual ~RbdWatchCtx() {}
   virtual void notify(uint8_t opcode, uint64_t ver, bufferlist& bl) {
-    cout << name << " got notification opcode=" << (int)opcode << " ver=" << ver << " bl.length=" << bl.length() << std::endl;
+    cout << name << " got notification opcode=" << (int)opcode << " ver="
+	 << ver << " bl.length=" << bl.length() << std::endl;
   }
 };
 
@@ -1131,7 +1367,8 @@ static int do_watch(librados::IoCtx& pp, const char *imgname)
   return 0;
 }
 
-static int do_kernel_add(const char *poolname, const char *imgname, const char *snapname)
+static int do_kernel_add(const char *poolname, const char *imgname,
+			 const char *snapname)
 {
   MonMap monmap;
   int r = monmap.build_initial(g_ceph_context, cerr);
@@ -1189,13 +1426,27 @@ static int do_kernel_add(const char *poolname, const char *imgname, const char *
     oss << " " << snapname;
   }
 
+  // modprobe the rbd module if /sys/bus/rbd doesn't exist
+  struct stat sb;
+  if ((stat("/sys/bus/rbd", &sb) < 0) || (!S_ISDIR(sb.st_mode))) {
+    r = system("/sbin/modprobe rbd");
+    if (r) {
+      if (r < 0)
+        cerr << "rbd: error executing modprobe as shell command!" << std::endl;
+      else
+        cerr << "rbd: modprobe rbd failed! (" << r << ")" <<std::endl;
+      return r;
+    }
+  }
+
   // write to /sys/bus/rbd/add
   int fd = open("/sys/bus/rbd/add", O_WRONLY);
   if (fd < 0) {
     r = -errno;
     if (r == -ENOENT) {
       cerr << "rbd: /sys/bus/rbd/add does not exist!" << std::endl
-	   << "Did you run 'modprobe rbd' or is your rbd module too old?" << std::endl;
+	   << "Did you run 'modprobe rbd' or is your rbd module too old?"
+	   << std::endl;
     }
     return r;
   }
@@ -1204,6 +1455,18 @@ static int do_kernel_add(const char *poolname, const char *imgname, const char *
   r = safe_write(fd, add.c_str(), add.size());
   close(fd);
 
+  // let udevadm do its job before we return
+  if (udevadm_settle) {
+    r = system("/sbin/udevadm settle");
+    if (r) {
+      if (r < 0)
+        cerr << "rbd: error executing udevadm as shell command!" << std::endl;
+      else
+        cerr << "rbd: '/sbin/udevadm settle' failed! (" << r << ")" <<std::endl;
+      return r;
+    }
+  }
+
   return r;
 }
 
@@ -1237,10 +1500,12 @@ void do_closedir(DIR *dp)
     closedir(dp);
 }
 
-static int do_kernel_showmapped()
+static int do_kernel_showmapped(Formatter *f)
 {
   int r;
   bool have_output = false;
+  TextTable tbl;
+
   const char *devices_path = "/sys/bus/rbd/devices";
   std::tr1::shared_ptr<DIR> device_dir(opendir(devices_path), do_closedir);
   if (!device_dir.get()) {
@@ -1259,12 +1524,15 @@ static int do_kernel_showmapped()
     return r;
   }
 
-  TextTable tbl;
-  tbl.define_column("id", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("pool", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("image", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("snap", TextTable::LEFT, TextTable::LEFT);
-  tbl.define_column("device", TextTable::LEFT, TextTable::LEFT);
+  if (f) {
+    f->open_object_section("devices");
+  } else {
+    tbl.define_column("id", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("pool", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("image", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("snap", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("device", TextTable::LEFT, TextTable::LEFT);
+  }
 
   do {
     if (strcmp(dent->d_name, ".") == 0 || strcmp(dent->d_name, "..") == 0)
@@ -1302,12 +1570,27 @@ static int do_kernel_showmapped()
       continue;
     }
 
-    tbl << dent->d_name << pool << name << snap << dev << TextTable::endrow;
+    if (f) {
+      f->open_object_section(dent->d_name);
+      f->dump_string("pool", pool);
+      f->dump_string("name", name);
+      f->dump_string("snap", snap);
+      f->dump_string("device", dev);
+      f->close_section();
+    } else {
+      tbl << dent->d_name << pool << name << snap << dev << TextTable::endrow;
+    }
     have_output = true;
 
   } while ((dent = readdir(device_dir.get())));
-  if (have_output)
-    cout << tbl;
+
+  if (f) {
+    f->close_section();
+    f->flush(cout);
+  } else {
+    if (have_output)
+      cout << tbl;
+  }
 
   return 0;
 }
@@ -1395,6 +1678,19 @@ static int do_kernel_rm(const char *dev)
   }
 
   r = close(fd);
+
+  // let udevadm finish, if present
+  if (udevadm_settle){
+    r = system("/sbin/udevadm settle");
+    if (r) {
+      if (r < 0)
+        cerr << "rbd: error executing udevadm as shell command!" << std::endl;
+      else
+        cerr << "rbd: '/sbin/udevadm settle' failed! (" << r << ")" <<std::endl;
+      return r;
+    }
+  }
+
   if (r < 0)
     r = -errno;
   return r;
@@ -1544,14 +1840,15 @@ int main(int argc, const char **argv)
   const char *poolname = NULL;
   uint64_t size = 0;  // in bytes
   int order = 0;
-  bool format_specified = false;
+  bool format_specified = false, output_format_specified = false;
   int format = 1;
   uint64_t features = RBD_FEATURE_LAYERING;
   const char *imgname = NULL, *snapname = NULL, *destname = NULL,
     *dest_poolname = NULL, *dest_snapname = NULL, *path = NULL,
     *devpath = NULL, *lock_cookie = NULL, *lock_client = NULL,
-    *lock_tag = NULL;
+    *lock_tag = NULL, *output_format = "plain";
   bool lflag = false;
+  int pretty_format = 0;
   long long stripe_unit = 0, stripe_count = 0;
   long long bench_io_size = 4096, bench_io_threads = 16, bench_bytes = 1 << 30;
 
@@ -1571,7 +1868,7 @@ int main(int argc, const char **argv)
     } else if (ceph_argparse_flag(args, i, "--new-format", (char*)NULL)) {
       format = 2;
       format_specified = true;
-    } else if (ceph_argparse_withint(args, i, &format, &err, "--format",
+    } else if (ceph_argparse_withint(args, i, &format, &err, "--image-format",
 				     (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << "rbd: " << err.str() << std::endl;
@@ -1618,6 +1915,22 @@ int main(int argc, const char **argv)
       imgname = strdup(val.c_str());
     } else if (ceph_argparse_witharg(args, i, &val, "--shared", (char *)NULL)) {
       lock_tag = strdup(val.c_str());
+    } else if (ceph_argparse_flag(args, i, "--no-settle", (char *)NULL)) {
+      udevadm_settle = false;
+    } else if (ceph_argparse_witharg(args, i, &val, "--format", (char *) NULL)) {
+      std::string err;
+      long long ret = strict_strtoll(val.c_str(), 10, &err);
+      if (err.empty()) {
+	format = ret;
+	format_specified = true;
+	cerr << "rbd: using --format for specifying the rbd image format is"
+	     << " deprecated, use --image-format instead"
+	     << std::endl;
+      } else {
+	output_format = strdup(val.c_str());
+	output_format_specified = true;
+      }
+    } else if (ceph_argparse_binary_flag(args, i, &pretty_format, NULL, "--pretty-format", (char*)NULL)) {
     } else {
       ++i;
     }
@@ -1719,14 +2032,31 @@ if (!set_conf_param(v, p1, p2, p3)) { \
   }
 
   if (format_specified && opt_cmd != OPT_IMPORT && opt_cmd != OPT_CREATE) {
-    cerr << "rbd: format can only be set when "
+    cerr << "rbd: image format can only be set when "
 	 << "creating or importing an image" << std::endl;
     return EXIT_FAILURE;
   }
 
+  if (pretty_format && !strcmp(output_format, "plain")) {
+    cerr << "rbd: --pretty-format only works when --format is json or xml"
+	 << std::endl;
+    return EXIT_FAILURE;
+  }
+
+  boost::scoped_ptr<Formatter> formatter;
+  if (output_format_specified && opt_cmd != OPT_SHOWMAPPED &&
+      opt_cmd != OPT_INFO && opt_cmd != OPT_LIST && opt_cmd != OPT_SNAP_LIST &&
+      opt_cmd != OPT_LOCK_LIST && opt_cmd != OPT_CHILDREN) {
+    cerr << "rbd: command doesn't use output formatting"
+	 << std::endl;
+    return EXIT_FAILURE;
+  } else if (get_outfmt(output_format, pretty_format, &formatter) < 0) {
+    return EXIT_FAILURE;
+  }
+
   if (format_specified) {
     if (format < 1 || format > 2) {
-      cerr << "rbd: format must be 1 or 2" << std::endl;
+      cerr << "rbd: image format must be 1 or 2" << std::endl;
       return EXIT_FAILURE;
     }
   }
@@ -1760,8 +2090,8 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     return EXIT_FAILURE;
   }
 
-  if (opt_cmd != OPT_LIST && opt_cmd != OPT_IMPORT && opt_cmd != OPT_UNMAP && opt_cmd != OPT_SHOWMAPPED &&
-      !imgname) {
+  if (opt_cmd != OPT_LIST && opt_cmd != OPT_IMPORT && opt_cmd != OPT_UNMAP &&
+      opt_cmd != OPT_SHOWMAPPED && !imgname) {
     cerr << "rbd: image name was not specified" << std::endl;
     return EXIT_FAILURE;
   }
@@ -1793,7 +2123,8 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     return EXIT_FAILURE;
   }
 
-  set_pool_image_name(dest_poolname, destname, (char **)&dest_poolname, (char **)&destname, (char **)&dest_snapname);
+  set_pool_image_name(dest_poolname, destname, (char **)&dest_poolname,
+		      (char **)&destname, (char **)&dest_snapname);
 
   if (opt_cmd == OPT_IMPORT) {
     if (poolname && dest_poolname) {
@@ -1917,7 +2248,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
 
   switch (opt_cmd) {
   case OPT_LIST:
-    r = do_list(rbd, io_ctx, lflag);
+    r = do_list(rbd, io_ctx, lflag, formatter.get());
     if (r < 0) {
       switch (r) {
       case -ENOENT:
@@ -1938,11 +2269,13 @@ if (!set_conf_param(v, p1, p2, p3)) { \
       return EXIT_FAILURE;
     }
     if ((stripe_unit && !stripe_count) || (!stripe_unit && stripe_count)) {
-      cerr << "must specify both (or neither) of stripe-unit and stripe-count" << std::endl;
+      cerr << "must specify both (or neither) of stripe-unit and stripe-count"
+	   << std::endl;
       usage();
       return EXIT_FAILURE;
     }
-    r = do_create(rbd, io_ctx, imgname, size, &order, format, features, stripe_unit, stripe_count);
+    r = do_create(rbd, io_ctx, imgname, size, &order, format, features,
+		  stripe_unit, stripe_count);
     if (r < 0) {
       cerr << "rbd: create error: " << cpp_strerror(-r) << std::endl;
       return EXIT_FAILURE;
@@ -1981,7 +2314,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     break;
 
   case OPT_INFO:
-    r = do_show_info(imgname, image, snapname);
+    r = do_show_info(imgname, image, snapname, formatter.get());
     if (r < 0) {
       cerr << "rbd: info: " << cpp_strerror(-r) << std::endl;
       return EXIT_FAILURE;
@@ -2022,7 +2355,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
       cerr << "rbd: snap list requires an image parameter" << std::endl;
       return EXIT_FAILURE;
     }
-    r = do_list_snaps(image);
+    r = do_list_snaps(image, formatter.get());
     if (r < 0) {
       cerr << "rbd: failed to list snapshots: " << cpp_strerror(-r)
 	   << std::endl;
@@ -2111,7 +2444,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     break;
 
   case OPT_CHILDREN:
-    r = do_list_children(image);
+    r = do_list_children(image, formatter.get());
     if (r < 0) {
       cerr << "rbd: listing children failed: " << cpp_strerror(-r) << std::endl;
       return EXIT_FAILURE;
@@ -2176,7 +2509,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     break;
 
   case OPT_SHOWMAPPED:
-    r = do_kernel_showmapped();
+    r = do_kernel_showmapped(formatter.get());
     if (r < 0) {
       cerr << "rbd: showmapped failed: " << cpp_strerror(-r) << std::endl;
       return EXIT_FAILURE;
@@ -2184,7 +2517,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     break;
 
   case OPT_LOCK_LIST:
-    r = do_lock_list(image);
+    r = do_lock_list(image, formatter.get());
     if (r < 0) {
       cerr << "rbd: listing locks failed: " << cpp_strerror(r) << std::endl;
       return EXIT_FAILURE;
