@@ -1213,7 +1213,7 @@ int main(int argc, char **argv)
 
       info.subusers[subuser] = u;
     }
-    if ((err = rgw_store_user_info(store, info, false)) < 0) {
+    if ((err = rgw_store_user_info(store, info, &old_info, false)) < 0) {
       cerr << "error storing user info: " << cpp_strerror(-err) << std::endl;
       break;
     }
@@ -1239,7 +1239,7 @@ int main(int argc, char **argv)
         keys_map->erase(kiter);
       }
     }
-    if ((err = rgw_store_user_info(store, info, false)) < 0) {
+    if ((err = rgw_store_user_info(store, info, &old_info, false)) < 0) {
       cerr << "error storing user info: " << cpp_strerror(-err) << std::endl;
       break;
     }
@@ -1265,7 +1265,7 @@ int main(int argc, char **argv)
       } else {
         rgw_remove_key_index(store, kiter->second);
         keys_map->erase(kiter);
-        if ((err = rgw_store_user_info(store, info, false)) < 0) {
+        if ((err = rgw_store_user_info(store, info, &old_info, false)) < 0) {
           cerr << "error storing user info: " << cpp_strerror(-err) << std::endl;
           break;
         }
@@ -1455,7 +1455,7 @@ int main(int argc, char **argv)
 
   if (opt_cmd == OPT_LOG_SHOW || opt_cmd == OPT_LOG_RM) {
     if (object.empty() && (date.empty() || bucket_name.empty() || bucket_id.empty())) {
-      cerr << "object or (at least one of date, bucket, bucket-id) were not specified" << std::endl;
+      cerr << "specify an object or a date, bucket and bucket-id" << std::endl;
       return usage();
     }
 
@@ -1660,7 +1660,7 @@ next:
 
     int ret;
     info.suspended = disable;
-    ret = rgw_store_user_info(store, info, false);
+    ret = rgw_store_user_info(store, info, &old_info, false);
     if (ret < 0) {
       cerr << "ERROR: failed to store user info user=" << user_id << " ret=" << ret << std::endl;
       return 1;
